/*
	File:		CappuccinoEmbedding.cpp
	
	Contents:	Stub functions to be implemented for embedding support.
	
				If you wish to add embedding support to a part, you must 
				implement the functions in this file.  If you do not wish 
				to support this feature, you can save a little space by removing 
				these functions and the similarly-named functions in the SOM 
				class.
	
	Written by:	Troy Gaul
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (eg. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- Cappuccino Includes --

#ifndef _CAPPUCCINO_
#include "Cappuccino.h"
#endif

#ifndef _CAPPUCCINODEF_
#include "CappuccinoDef.h"
#endif

#ifndef _CAPPUCCINOGLOBALS_
#include "CappuccinoGlobals.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

// -- OpenDoc Utilities --

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif


//------------------------------------------------------------------------------
// Method:		RequestEmbeddedFrame
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODFrame* Cappuccino::RequestEmbeddedFrame( Environment*		ev,
										   ODFrame*			containingFrame,
										   ODFrame*			baseFrame,
										   ODShape*			frameShape,
										   ODPart*			embeddedPart,
										   ODTypeToken		viewType,
										   ODTypeToken		presentation,
										   ODBoolean		isOverlaid )
{
	SOM_Trace("Cappuccino","RequestEmbeddedFrame");

	THROW(kODErrCannotEmbed);

	return kODNULL;
}

//------------------------------------------------------------------------------
// Method:		RemoveEmbeddedFrame
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

void Cappuccino::RemoveEmbeddedFrame( Environment*		ev,
									  ODFrame*			embeddedFrame )
{
	SOM_Trace("Cappuccino","RemoveEmbeddedFrame");

	THROW(kODErrCannotEmbed);
}

//------------------------------------------------------------------------------
// Method:		RequestFrameShape
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODShape* Cappuccino::RequestFrameShape( Environment*	ev,
										ODFrame*		embeddedFrame,
										ODShape*		frameShape )
{
	SOM_Trace("Cappuccino","RequestFrameShape");

	THROW(kODErrCannotEmbed);

	return kODNULL;
}

//------------------------------------------------------------------------------
// Method:		UsedShapeChanged
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

void Cappuccino::UsedShapeChanged( Environment*		ev,
								   ODFrame*			embeddedFrame )
{
	SOM_Trace("Cappuccino","UsedShapeChanged");

	THROW(kODErrCannotEmbed);
}

//------------------------------------------------------------------------------
// Method:		AdjustBorderShape
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODShape* Cappuccino::AdjustBorderShape( Environment*	ev,
										ODFacet*		embeddedFacet,
										ODShape*		shape )
{
	SOM_Trace("Cappuccino","AdjustBorderShape");

	THROW(kODErrCannotEmbed);

	return kODNULL;
}

//------------------------------------------------------------------------------
// Method:		AcquireContainingPartProperties
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODStorageUnit* Cappuccino::AcquireContainingPartProperties( Environment*	ev,
															ODFrame*		frame )
{
	SOM_Trace("Cappuccino","GetContainingPartProperties");

	THROW(kODErrCannotEmbed);

	return kODNULL;
}

//------------------------------------------------------------------------------
// Method:		RevealFrame
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODBoolean Cappuccino::RevealFrame( Environment*		ev,
								   ODFrame*			embeddedFrame,
								   ODShape*			revealShape )
{
	SOM_Trace("Cappuccino","RevealFrame");

	THROW(kODErrCannotEmbed);

	return kODFalse;
}

//------------------------------------------------------------------------------
// Method:		EmbeddedFrameSpec
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

void Cappuccino::EmbeddedFrameSpec( Environment*	ev,
									ODFrame*		embeddedFrame,
									ODObjectSpec*	spec )
{
	SOM_Trace("Cappuccino","EmbeddedFrameSpec");

	THROW(kODErrCannotEmbed);
}

//------------------------------------------------------------------------------
// Method:		CreateEmbeddedFramesIterator
// Origin:		ODPart
//
// Description:	This method is called ...
//------------------------------------------------------------------------------

ODEmbeddedFramesIterator* Cappuccino::CreateEmbeddedFramesIterator( Environment*	ev,
																	ODFrame*		frame )
{
	SOM_Trace("Cappuccino","CreateEmbeddedFramesIterator");

	THROW(kODErrCannotEmbed);

	return kODNULL;
}


