/*
	File:		CappuccinoSettingsExt.cpp

	Contains:	Settings extension class implementation.

	Written by:	Troy Gaul

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/


// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (eg. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- Cappuccino Includes --

#ifndef _CAPPUCCINOSETTINGSEXT_
#include "CappuccinoSettingsExt.h"
#endif

#ifndef _CAPPUCCINO_
#include "Cappuccino.h"
#endif

#ifndef SOM_AppleComputer_som_Cappuccino_xh
#include "som_Cappuccino.xh"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif


#pragma segment CappuccinoSettingsExtension

//==============================================================================
// CappuccinoSettingsExtension
//==============================================================================

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	
//------------------------------------------------------------------------------

CappuccinoSettingsExtension::CappuccinoSettingsExtension()
{
    SOM_Trace("CappuccinoSettingsExtension","Constructor");
    
    fOwner = kODNULL;
}

//------------------------------------------------------------------------------
// Method:		Destructor
//
// Description:	
//------------------------------------------------------------------------------

CappuccinoSettingsExtension::~CappuccinoSettingsExtension()
{
    SOM_Trace("CappuccinoSettingsExtension","Destructor");
}

//------------------------------------------------------------------------------
// Method:		InitSettingsExtension
//
// Description:	
//------------------------------------------------------------------------------

void CappuccinoSettingsExtension::InitSettingsExtension( Environment*	ev,
														 ODPart*		owner )
{
    SOM_Trace("CappuccinoSettingsExtension","InitSettingsExtension");
	
	// Here we get the real part, which means that we get the actual ODPart
	// that the part wrapper points to.  We know that this will be an
	// object of the AppleComputer_som_Cappuccino class, so we cast it thusly.
	// From this object, we use our GetImplementation function, which
	// returns a pointer to our wrapped C++ object so that we can call it
	// directly.  This is safe because the extension and the part will be in
	// the same address space and machine.
	fOwner = ((AppleComputer_som_Cappuccino*) owner->GetRealPart(ev))
										->GetImplementation(ev);
	
	owner->ReleaseRealPart(ev);
}

//------------------------------------------------------------------------------
// Method:		ShowSettings
//
// Description:	
//------------------------------------------------------------------------------

void CappuccinoSettingsExtension::ShowSettings( Environment*	ev,
												ODFacet*		facet )
{
    SOM_Trace("CappuccinoSettingsExtension","ShowSettings");
	
	fOwner->ShowSettingsDialog(ev, facet->GetFrame(ev));
}

