/*
	File:		som_Cappuccino.cpp

	Contains:	Sample Part's SOM based part public interface implementation

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
	
	----------------------------------------------------------------
	
	Notes:		Cappuccino is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.
*/

// Notification that this is a SOM source file
#define AppleComputer_som_Cappuccino_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- Cappuccino Includes --

#ifndef SOM_AppleComputer_som_Cappuccino_xih
#include "som_Cappuccino.xih"
#endif

#ifndef _CAPPUCCINO_
#include "Cappuccino.h"
#endif

#ifndef _CAPPUCCINODEF_
#include "CappuccinoDef.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#pragma segment somCappuccino

//====================================================================
// som_Cappuccino
//====================================================================

SOM_Scope void  SOMLINK som_Cappuccino__somInit(AppleComputer_som_Cappuccino *somSelf)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically. Thus, there is no need to
	// call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_Cappuccino__somUninit(AppleComputer_som_Cappuccino *somSelf)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__somUninit");

	delete _fPart;
}

//------------------------------------------------------------------------------
// Initialization
//------------------------------------------------------------------------------
#pragma mark  Initialization 

SOM_Scope void  SOMLINK som_Cappuccino__InitPart(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__InitPart");

	SOM_TRY
		// Create the real C++ Cappuccino.		
		_fPart = Cappuccino::CreatePartObject();

		AppleComputer_som_Cappuccino_parent_ODPart_InitPart(somSelf, ev, storageUnit, partWrapper);
		_fPart->InitPart(ev, storageUnit, partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__InitPartFromStorage(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__InitPartFromStorage");

	SOM_TRY
		// Create the real C++ Cappuccino.		
		_fPart = Cappuccino::CreatePartObject();

		AppleComputer_som_Cappuccino_parent_ODPart_InitPartFromStorage(somSelf, ev, storageUnit, partWrapper);
		_fPart->InitPartFromStorage(ev, storageUnit, partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// ODPersistentObject
//------------------------------------------------------------------------------
#pragma mark  ODPersistentObject 

SOM_Scope void  SOMLINK som_Cappuccino__Externalize(AppleComputer_som_Cappuccino *somSelf, Environment *ev)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Externalize");

	SOM_TRY
		AppleComputer_som_Cappuccino_parent_ODPart_Externalize(somSelf, ev);
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODSize  SOMLINK som_Cappuccino__Purge(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODSize size)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Purge");

	ODSize result;

	SOM_TRY
		result = _fPart->Purge(ev, size);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__Release(AppleComputer_som_Cappuccino *somSelf, Environment *ev)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Release");

	SOM_TRY
		AppleComputer_som_Cappuccino_parent_ODPart_Release(somSelf, ev);
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__ReleaseAll(AppleComputer_som_Cappuccino *somSelf, Environment *ev)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
		AppleComputer_som_Cappuccino_parent_ODPart_ReleaseAll(somSelf, ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__CloneInto(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CloneInto");

	SOM_TRY
		AppleComputer_som_Cappuccino_parent_ODPart_CloneInto(somSelf, ev, key, toSU, scope);
		_fPart->CloneInto(ev, key, toSU, scope);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Extensions
//------------------------------------------------------------------------------
#pragma mark  Extensions 

SOM_Scope ODBoolean  SOMLINK som_Cappuccino__HasExtension(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODType extensionName)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__HasExtension");

	ODBoolean result = kODFalse;
	
	SOM_TRY
		result = _fPart->HasExtension(ev, extensionName);
		
		// If we don't support this extension, see if one of our parent classes might.
		if (result == kODFalse)
			result = AppleComputer_som_Cappuccino_parent_ODPart_HasExtension(somSelf, ev, extensionName);
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

SOM_Scope ODExtension*  SOMLINK som_Cappuccino__AcquireExtension(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODType extensionName)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__GetExtension");

	ODExtension* result = kODNULL;

	SOM_TRY
		result = _fPart->AcquireExtension(ev, extensionName);

		// If we don't support this extension, see if one of our parent classes might.
		if (result == kODNULL)
			result = AppleComputer_som_Cappuccino_parent_ODPart_AcquireExtension(somSelf, ev, extensionName);
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__ReleaseExtension(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODExtension* extension)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
    AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ReleaseExtension");

	SOM_TRY
		// If we don't support this extension, one of our parent classes might.
		if (!_fPart->ReleaseExtension(ev, extension))
			AppleComputer_som_Cappuccino_parent_ODPart_ReleaseExtension(somSelf, ev, extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Layout
//------------------------------------------------------------------------------
#pragma mark  Layout 

SOM_Scope void  SOMLINK som_Cappuccino__DisplayFrameAdded(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__DisplayFrameRemoved(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__DisplayFrameConnected(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__DisplayFrameClosed(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__AttachSourceFrame(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev, frame, sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__FrameShapeChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__ViewTypeChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__PresentationChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__PresentationChanged");

	SOM_TRY
		_fPart->PresentationChanged(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__SequenceChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__SequenceChanged");

	SOM_TRY
		_fPart->SequenceChanged(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__LinkStatusChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__LinkStatusChanged");

	SOM_TRY
		_fPart->LinkStatusChanged(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__ContainingPartPropertiesUpdated(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnit* propertyUnit)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ContainingPartPropertiesChanged");

	SOM_TRY
		_fPart->ContainingPartPropertiesUpdated(ev, frame, propertyUnit);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_Cappuccino__ReadPartInfo(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ReadPartInfo");

	ODInfoType result;

	SOM_TRY
		result = _fPart->ReadPartInfo(ev, frame, storageUnitView);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__WritePartInfo(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev, partInfo, storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__ClonePartInfo(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scopeFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev, key, partInfo, storageUnitView, scopeFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODID  SOMLINK som_Cappuccino__Open(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Open");

	ODID result;

	SOM_TRY
		result = _fPart->Open(ev, frame);
	SOM_CATCH_ALL
		result = kODNULLID;
	SOM_ENDTRY

	return result;
}

//------------------------------------------------------------------------------
// Imaging
//------------------------------------------------------------------------------
#pragma mark  Imaging 

SOM_Scope void  SOMLINK som_Cappuccino__Draw(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Draw");

	SOM_TRY
		_fPart->Draw(ev, facet, invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__FacetAdded(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev, facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__FacetRemoved(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev, facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__GeometryChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev, facet, clipShapeChanged, externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__HighlightChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev, facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__CanvasChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CanvasChanged");

	SOM_TRY
		_fPart->CanvasChanged(ev, facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__CanvasUpdated(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODCanvas* canvas)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CanvasUpdated");

	SOM_TRY
		_fPart->CanvasUpdated(ev, canvas);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODULong  SOMLINK som_Cappuccino__GetPrintResolution(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__GetPrintResolution");
	
	ODULong result = 72;
	
	SOM_TRY
		_fPart->GetPrintResolution(ev, frame);
	SOM_CATCH_ALL
		result = 72;
	SOM_ENDTRY

	return result;
}

//------------------------------------------------------------------------------
// Activation
//------------------------------------------------------------------------------
#pragma mark  Activation 

SOM_Scope ODBoolean  SOMLINK som_Cappuccino__BeginRelinquishFocus(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__BeginRelinquishFocus");

	ODBoolean result;

	SOM_TRY
		result = _fPart->BeginRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__CommitRelinquishFocus(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__AbortRelinquishFocus(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__FocusAcquired(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev, focus, ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__FocusLost(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev, focus, ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Persistance
//------------------------------------------------------------------------------
#pragma mark  Persistance 

SOM_Scope void  SOMLINK som_Cappuccino__ExternalizeKinds(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODTypeList* kindset)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ExternalizeKinds");

	SOM_TRY
		_fPart->ExternalizeKinds(ev, kindset);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__ChangeKind(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODType kind)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__ChangeKind");

	SOM_TRY
		_fPart->ChangeKind(ev, kind);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// UI Events
//------------------------------------------------------------------------------
#pragma mark  UI Events 

SOM_Scope ODBoolean  SOMLINK som_Cappuccino__HandleEvent(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__HandleEvent");

	ODBoolean result;

	SOM_TRY
		result = _fPart->HandleEvent(ev, event, frame, facet, eventInfo);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__AdjustMenus(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Undo
//------------------------------------------------------------------------------
#pragma mark  Undo 

SOM_Scope void  SOMLINK som_Cappuccino__UndoAction(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODActionData* actionState)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__UndoAction");

	SOM_TRY
		_fPart->UndoAction(ev, actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__RedoAction(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODActionData* actionState)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RedoAction");

	SOM_TRY
		_fPart->RedoAction(ev, actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__DisposeActionState(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODActionData* actionState,
		ODDoneState doneState)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DisposeActionState");

	SOM_TRY
		_fPart->DisposeActionState(ev, actionState, doneState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Drag And Drop
//------------------------------------------------------------------------------
#pragma mark  Drag And Drop 

SOM_Scope void  SOMLINK som_Cappuccino__FulfillPromise(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODStorageUnitView* promiseSUView)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__FulfillPromise");

	SOM_TRY
		_fPart->FulfillPromise(ev, promiseSUView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__DropCompleted(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODPart* destPart,
		ODDropResult dropResult)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DropCompleted");

	SOM_TRY
		_fPart->DropCompleted(ev, destPart, dropResult);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODDragResult  SOMLINK som_Cappuccino__DragEnter(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DragEnter");
	
	ODDragResult result;

	SOM_TRY
		result = _fPart->DragEnter(ev, dragInfo, facet, where);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODDragResult  SOMLINK som_Cappuccino__DragWithin(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DragWithin");
	
	ODDragResult result;

	SOM_TRY
		result = _fPart->DragWithin(ev, dragInfo, facet, where);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__DragLeave(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* facet,
		ODPoint* where)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__DragLeave");

	SOM_TRY
		_fPart->DragLeave(ev, facet, where);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODDropResult  SOMLINK som_Cappuccino__Drop(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODDragItemIterator* dropInfo,
		ODFacet* facet,
		ODPoint* where)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__Drop");

	ODDropResult result;

	SOM_TRY
		result = _fPart->Drop(ev, dropInfo, facet, where);
	SOM_CATCH_ALL
		result = kODDropFail;
	SOM_ENDTRY

	return result;
}

//------------------------------------------------------------------------------
// Linking
//------------------------------------------------------------------------------
#pragma mark  Linking 

SOM_Scope ODLinkSource*  SOMLINK som_Cappuccino__CreateLink(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODByteArray* data)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CreateLink");

	ODLinkSource* result;
	
	SOM_TRY
		result = _fPart->CreateLink(ev, data);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__LinkUpdated(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODLink* link,
		ODUpdateID updateID)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__LinkUpdated");

	SOM_TRY
		_fPart->LinkUpdated(ev, link, updateID);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__RevealLink(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODLinkSource* linkSource)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RevealLink");

	SOM_TRY
		_fPart->RevealLink(ev, linkSource);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_Cappuccino__EmbeddedFrameUpdated(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame,
		ODUpdateID change)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__EmbeddedFrameChanged");

	SOM_TRY
		_fPart->EmbeddedFrameUpdated(ev, frame, change);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_Cappuccino__EditInLinkAttempted(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
    AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__EditInLinkAttempted");

	ODBoolean result;
	
	SOM_TRY
		result = _fPart->EditInLinkAttempted(ev, frame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

//------------------------------------------------------------------------------
// Embedding
//------------------------------------------------------------------------------
#pragma mark  Embedding 

SOM_Scope ODFrame*  SOMLINK som_Cappuccino__RequestEmbeddedFrame(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* containingFrame,
		ODFrame* baseFrame,
		ODShape* frameShape,
		ODPart* embeddedPart,
		ODTypeToken viewType,
		ODTypeToken presentation,
		ODBoolean isOverlaid)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RequestEmbeddedFrame");

	ODFrame* result;
	
	SOM_TRY
		result = _fPart->RequestEmbeddedFrame(ev, containingFrame, baseFrame,
											  frameShape, embeddedPart, viewType,
											  presentation, isOverlaid);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__RemoveEmbeddedFrame(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* embeddedFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RemoveEmbeddedFrame");

	SOM_TRY
		_fPart->RemoveEmbeddedFrame(ev, embeddedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODShape*  SOMLINK som_Cappuccino__RequestFrameShape(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* embeddedFrame,
		ODShape* frameShape)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RequestFrameShape");

	ODShape* result;
	
	SOM_TRY
		result = _fPart->RequestFrameShape(ev, embeddedFrame, frameShape);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__UsedShapeChanged(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* embeddedFrame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__UsedShapeChanged");

	SOM_TRY
		_fPart->UsedShapeChanged(ev, embeddedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODShape*  SOMLINK som_Cappuccino__AdjustBorderShape(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFacet* embeddedFacet,
		ODShape* shape)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__AdjustBorderShape");

	ODShape* result;
	
	SOM_TRY
		result = _fPart->AdjustBorderShape(ev, embeddedFacet, shape);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODStorageUnit*  SOMLINK som_Cappuccino__AcquireContainingPartProperties(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__GetContainingPartProperties");

	ODStorageUnit* result;
	
	SOM_TRY
		result = _fPart->AcquireContainingPartProperties(ev, frame);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODBoolean  SOMLINK som_Cappuccino__RevealFrame(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* embeddedFrame,
		ODShape* revealShape)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__RevealFrame");

	ODBoolean result;
	
	SOM_TRY
		result = _fPart->RevealFrame(ev, embeddedFrame, revealShape);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_Cappuccino__EmbeddedFrameSpec(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* embeddedFrame,
		ODObjectSpec* spec)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__EmbeddedFrameSpec");

	SOM_TRY
		_fPart->EmbeddedFrameSpec(ev, embeddedFrame, spec);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODEmbeddedFramesIterator*  SOMLINK som_Cappuccino__CreateEmbeddedFramesIterator(AppleComputer_som_Cappuccino *somSelf, Environment *ev,
		ODFrame* frame)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__CreateEmbeddedFramesIterator");

	ODEmbeddedFramesIterator* result;
	
	SOM_TRY
		result = _fPart->CreateEmbeddedFramesIterator(ev, frame);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

//------------------------------------------------------------------------------
// Cappuccino-specific
//------------------------------------------------------------------------------
#pragma mark  Cappuccino-specific 

SOM_Scope Cappuccino*  SOMLINK som_Cappuccino__GetImplementation(AppleComputer_som_Cappuccino *somSelf, Environment *ev)
{
	AppleComputer_som_CappuccinoData *somThis = AppleComputer_som_CappuccinoGetData(somSelf);
	AppleComputer_som_CappuccinoMethodDebug("AppleComputer_som_Cappuccino","som_Cappuccino__GetImplementation");
	
	return _fPart;
}
