/*
	File:		DrawEditor.r

	Contains:	DrawEditor Resources

	Written by:	Dave Stafford based on SamplePart

	Copyright:	 1994,95 by Apple Computer, Inc., all rights reserved.
*/

#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

// -- MPW Rez Includes --

#include "Types.r"
#include "SysTypes.r"
#include "CodeFragmentTypes.r"

// -- OpenDoc Includes --

#ifndef __STDTYPES_R__
#include "ODTypes.r"
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include "StdDefs.r"
#endif

// -- DrawEditor Includes --

#ifndef _DRAWEDITORDEF_
#include "DrawEditorDef.h"
#endif

#ifndef _DRAWEDITORVERS_
#include "DrawEditorVers.h"
#endif

//-------------------------------------------------------------------------------------
// .rsrc resources
//-------------------------------------------------------------------------------------

// DrawEditor - This gets the resources from my .rsrc file (PICTs etc.)
include "DrawEditorOtherResources.rsrc";

//-------------------------------------------------------------------------------------
// Finder Version Resources
//-------------------------------------------------------------------------------------

// Part's Finder Version Resource

resource 'vers' (1) {
	currentMajorVersion,
	finderMinorVersion,
	developmentStage,
	preReleaseNumber,
	verUS,
	shortVersionStr,
	shortVersionStr",  Apple Computer, Inc. 1994-1995"
};

resource 'vers' (2) {
	currentMajorVersion,
	finderMinorVersion,
	developmentStage,
	preReleaseNumber,
	verUS,
	shortVersionStr,
	"OpenDoc Sample Code"
};

//-------------------------------------------------------------------------------------
// Code Fragment Resources
//-------------------------------------------------------------------------------------

resource 'cfrg' (0) {
	{	/* [1] */
#ifdef _68KBUILD_
		kMotorola,
#else
		kPowerPC,
#endif
		kFullLib,
		currentVersion,
		compatibleVersion,
		kDefaultStackSize,
		kNoAppSubFolder,
		kIsLib,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		kDrawEditorID,		/* This must be the class ID. */
		/* [2] */
#ifdef _68KBUILD_
		kMotorola,
#else
		kPowerPC,
#endif
		kFullLib,
		currentVersion,
		compatibleVersion,
		kDefaultStackSize,
		kNoAppSubFolder,
		kIsLib,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		kPartClassName		/* This must be the SOM class name */
							/* for this part.				   */
	}
};

//-------------------------------------------------------------------------------------
// NMAP Resources
//-------------------------------------------------------------------------------------

// Map a part's class id (module::classname)
// to a part's kind (unique content identifier)
//
resource kODNameMappings (kEditorKindMapId) {
	kODEditorKinds,
	{	/* array Types: 1 elements */
		/* [1] */
		kDrawEditorID,
		kODIsAnISOStringList
		{
			{	/* array ClassIDs: 1 elements */
				/* [1] */
				kDrawEditorKind
			}
		}
	}
};

// Map a part's class id (module::classname)
// to a human readable string.
//
resource kODNameMappings (kEditorUserStringMapId) {
	kODEditorUserString,
	{	/* array Types: 1 elements */
		/* [1] */
		kDrawEditorID,
		kODIsINTLText
		{
			smRoman,
			langEnglish,
			kDrawEditorEditorUserString
		}
	}
};

// Map a part's kind (unique content identifier)
// to a generic content category identifier.
//
resource kODNameMappings (kKindCategoryMapId) {
	kODKind,
	{	/* array Types: 1 elements */
		/* [1] */
		kDrawEditorKind,
		kODIsAnISOStringList
		{
			{	/* array ClassIDs: 1 elements */
				/* [1] */
				kODCategoryDrawing
			}
		}
	}
};

// Map a part's kind (unique content identifier)
// to a human readable string.
//
resource kODNameMappings (kKindUserStringMapId) {
	kODKindUserString,
	{	/* array Types: 1 elements */
		/* [1] */
		kDrawEditorKind,
		kODIsINTLText
		{
			smRoman,
			langEnglish,
			kDrawEditorKindUserString
		}
	}
};

// Map a part's kind (unique content identifier)
// to an old style MacOS Type (and/or ScrapType).
//
resource kODNameMappings (kOldMacOSTypeMapId) {
	kODKindOldMacOSType,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kDrawEditorKind,
		kODIsMacOSType {
			kDrawEditorDocumentOSType
		}
	}
};


//-------------------------------------------------------------------------------------
// String Lists
//-------------------------------------------------------------------------------------

// Menu strings
resource 'STR#' (kMenuStringResID, "Menu Items strings") {
	{	/* array StringArray: 3 elements */
		/* [1] */
		"About DrawEditor",
		/* [2] */
		"Hide Tool Palette",
		/* [3] */
		"Show Tool Palette",
		/* [4] */
		"Hide Color Palette",
		/* [5] */
		"Show Color Palette",
		/* [6] */
		"Undo Command",
		/* [7] */
		"Redo Command",
		/* [8] */
		"Undo New Shape",
		/* [9] */
		"Redo New Shape",
		/* [10] */
		"Undo Resize",
		/* [11] */
		"Redo Resize",
		/* [12] */
		"Undo Drag",
		/* [13] */
		"Redo Drag",
		/* [14] */
		"Undo Drop",
		/* [15] */
		"Redo Drop",
		/* [16] */
		"Undo Color Change",
		/* [17] */
		"Redo Color Change",
		/* [18] */
		"Undo Move Forward",
		/* [19] */
		"Redo Move Forward",
		/* [20] */
		"Undo Move Backward",
		/* [21] */
		"Redo Move Backward",
		/* [22] */
		"Undo Move First",
		/* [23] */
		"Redo Move First",
		/* [24] */
		"Undo Move Last",
		/* [25] */
		"Redo Move Last",
		/* [26] */
		"Undo Cut",
		/* [27] */
		"Redo Cut",
		/* [28] */
		"Undo Copy",
		/* [29] */
		"Redo Copy",
		/* [30] */
		"Undo Paste",
		/* [31] */
		"Redo Paste",
		/* [32] */
		"Undo Clear",
		/* [33] */
		"Redo Clear",
		/* [34] */
		"Undo Select All",
		/* [35] */
		"Redo Select All",
		/* [36] */
		"Undo Paste Link",
		/* [37] */
		"Redo Paste Link",
		/* [38] */
		"Link Info",
		/* [39] */
		"Undo Break Link",
		/* [40] */
		"Redo Break Link"
	}
};

// Error Text
resource 'STR#' (kErrorStringResID, "Error strings") {
	{	/* array StringArray: 6 elements */
		/* [1] */
		"DrawEditor was unable completely initi"
		"alize its internal structures.",
		/* [2] */
		"DrawEditor was unable to open the docu"
		"ment due to unforseen circumstances.",
		/* [3] */
		"DrawEditor was unable to open a part w"
		"indow due to unforseen circumstances.",
		/* [4] */
		"DrawEditor has been asked to remove a "
		"frame that does not belong to it, or an "
		"error occurred while removing the frame "
		"from internal storage.",
		/* [5] */
		"A window that DrawEditor created has b"
		"een deleted without its knowledge. This "
		"may generate a fatal error in the near f"
		"uture. Please close the document immedia"
		"tely to prevent data loss.",
		/* [6] */
		"DrawEditor encountered a fatal error w"
		"hile trying to save. Try saving again or"
		"closing the document."
	}
};


resource 'DITL' (20000) {
	{	/* array DITLarray: 1 elements */
		/* [1] */
		{0, 0, 196, 299},
		Picture {
			enabled,
			20009
		}
	}
};

resource 'DITL' (20001, "EditorError Alert") {
	{	/* array DITLarray: 5 elements */
		/* [1] */
		{153, 256, 173, 314},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{153, 185, 173, 243},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{11, 70, 124, 317},
		StaticText {
			disabled,
			"<< Use this for errors in the Editor, su"
			"ch as out of memory, unable to acquire s"
			"ome system resource, etc.>> \nUser errors"
			" use a standard alert appearance. See DI"
			"TL 129.>>"
		},
		/* [4] */
		{11, 19, 43, 51},
		Picture {
			disabled,
			5010
		},
		/* [5] */
		{149, 252, 177, 318},
		UserItem {
			disabled
		}
	}
};

resource 'DITL' (20002, "Drawing Size") {
	{	/* array DITLarray: 8 elements */
		/* [1] */
		{99, 121, 119, 179},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{99, 49, 119, 107},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 26, 183},
		StaticText {
			disabled,
			"Drawing Size: (in pages)"
		},
		/* [4] */
		{39, 83, 55, 104},
		EditText {
			enabled,
			"99"
		},
		/* [5] */
		{39, 34, 55, 78},
		StaticText {
			disabled,
			"Width:"
		},
		/* [6] */
		{66, 29, 82, 78},
		StaticText {
			disabled,
			"Height:"
		},
		/* [7] */
		{66, 83, 82, 104},
		EditText {
			enabled,
			"99"
		},
		/* [8] */
		{95, 117, 123, 183},
		UserItem {
			disabled
		}
	}
};

//-------------------------------------------------------------------------------------
// Finder Bundle Information
//-------------------------------------------------------------------------------------

resource 'BNDL' (kDocumentBundle, "Document BNDL") {
	kODShellSignature,
	0,
	{	/* array TypeArray: 2 elements */
		/* [1] */
		'FREF',
		{	/* array IDArray: 2 elements */
			/* [1] */
			0, kDocumentFREF,
			/* [2] */
			1, kStationeryFREF
		},
		/* [2] */
		'ICN#',
		{	/* array IDArray: 2 elements */
			/* [1] */
			0, kDocumentIcons,
			/* [2] */
			1, kStationeryIcons
		}
	}
};

resource 'BNDL' (kEditorBundle, "Editor BNDL") {
	kDrawEditorEditorOSType,
	0,
	{	/* array TypeArray: 2 elements */
		/* [1] */
		'FREF',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, kEditorFREF
		},
		/* [2] */
		'ICN#',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, kEditorIcons
		}
	}
};

resource 'BNDL' (kViewerBundle, "Viewer BNDL") {
	kDrawEditorViewerOSType,
	0,
	{	/* array TypeArray: 2 elements */
		/* [1] */
		'FREF',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, kViewerFREF
		},
		/* [2] */
		'ICN#',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, kViewerIcons
		}
	}
};

resource 'FREF' (kDocumentFREF) {
	kDrawEditorDocumentOSType,
	0,
	""
};

resource 'FREF' (kStationeryFREF) {
	kDrawEditorStationeryOSType,
	1,
	""
};

resource 'FREF' (kEditorFREF) {
	'shlb',
	0,
	""
};

resource 'FREF' (kViewerFREF) {
	'shlb',
	0,
	""
};

resource 'DLOG' (kErrorBoxID, "Error Box") {
	{57, 46, 244, 374},
	dBoxProc,
	visible,
	goAway,
	0x0,
	20001,
	"",
	alertPositionParentWindowScreen                                           
};


resource 'DLOG' (kAboutBoxID, "About Box") {
	{40, 40, 236, 339},
	dBoxProc,
	visible,
	goAway,
	0x0,
	20000,
	"",
	centerParentWindowScreen
};

resource 'DLOG' (kDrawingsizeDialogID, "Drawing Size") {
	{40, 40, 173, 233},
	dBoxProc,
	visible,
	goAway,
	0x0,
	20002,
	"",
	alertPositionParentWindowScreen
};


data 'odtm' (0, "Owner resource") {
	$"00"                                                 /* . */
};

resource 'MENU' (kPalettesMenuID, preload) {
	kPalettesMenuID,
	textMenuProc,
	allEnabled,
	enabled,
	"Palettes",
	{	/* array: 2 elements */
		/* [1] */
		"Show Tools", noIcon, noKey, noMark, plain,
		/* [2] */
		"Show Colors", noIcon, noKey, noMark, plain
	}
};

resource 'MENU' (kLayoutMenuID, preload) {
	kLayoutMenuID,
	textMenuProc,
	0x7FFFFFEF,
	enabled,
	"Layout",
	{	/* array: 6 elements */
		/* [1] */
		"Move Forward", noIcon, noKey, noMark, plain,
		/* [2] */
		"Move Front", noIcon, noKey, noMark, plain,
		/* [3] */
		"Move Backward", noIcon, noKey, noMark, plain,
		/* [4] */
		"Move Back", noIcon, noKey, noMark, plain,
	}
};

data 'DWDC' (1) {
	$"00"                                                 /* . */
};

data 'DWED' (0, "Owner resource") {
	$"00"                                                 /* . */
};

data 'WDEF' (kFloatWindowDefID, "Main", purgeable, preload) {
	$"600A 0000 5744 4546 0003 0002 4E56 FFDA"            /* `...WDEF....NV */
	$"48E7 3F78 0C6E 0006 000C 6E00 00F6 0C6E"            /* H?x.n....n...n */
	$"0000 000C 6B00 00EC 2055 2F10 0C78 3FFF"            /* ....k.. U/..x? */
	$"028E 53EE FFDE 6206 6100 00F4 6006 2F38"            /* .Sb.a..`./8 */
	$"09DE A873 47EE FFF8 3C2E 0012 0806 0001"            /* ިsG<....... */
	$"660C 7E0B 3D47 FFFC 426E FFFE 602A 2F0B"            /* f.~.=GBn`*.. */
	$"A88B 3E1B DE5B 5847 08C7 0000 7213 BE41"            /* >.[XG...r.A */
	$"6C0A 9247 E249 D36E FFF8 7E13 36C7 3007"            /* l.GIn~.60. */
	$"0440 000D E248 3680 42A7 A8D8 2D5F FFF4"            /* .@.H6B-_ */
	$"42A7 A8D8 2D5F FFDA 4A2E FFDE 672E 486E"            /* B-_J.g.Hn */
	$"FFE6 AA19 486E FFE0 AA1A 42A7 4267 2F2E"            /* .Hn.BBg/. */
	$"000E 486F 0006 AA42 544F 205F 2050 2068"            /* ..Ho..BTO _ P h */
	$"0008 2D48 FFF0 A029 2D50 FFEC 41EE 0008"            /* ..-H)-PA.. */
	$"2618 3018 2658 3A18 3C05 0245 0003 4290"            /* &.0.&X:.<..E..B */
	$"D040 41FA 003C D0FB 0038 4E90 2F2E FFF4"            /* @A.<.8N/. */
	$"A8D9 2F2E FFDA A8D9 4A2E FFDE 6712 206E"            /* /.ڨJ.g. n */
	$"FFF0 A02A 486E FFE0 AA15 486E FFE6 AA14"            /* *Hn.Hn. */
	$"A873 4CDF 1EFC 4E5E 205F 4FEF 000C 4ED0"            /* sL.N^ _O..N */
	$"0138 052A 0620 0046 00F6 06AC 0768 2F38"            /* .8.*. .F...h/8 */
	$"0D2C A873 2078 09DE 2278 0D2C 4868 0020"            /* ,s x"x,Hh.  */
	$"4868 003A 41E8 0030 43E9 0030 22D8 22D8"            /* Hh.:A.0C.0"" */
	$"32D8 5048 5049 303C 0014 32D8 51C8 FFFC"            /* 2PHPI0<..2Q */
	$"A89D A87C 4E75 0806 0003 6758 422B 0071"            /* |Nu....gXB+.q */
	$"4A78 028E 6B4E 7010 A322 6648 522B 0071"            /* Jx.kNp."fHR+.q */
	$"2748 0082 2850 504C 2255 28E9 FF8C 28E9"            /* 'H.(PPL"U(( */
	$"FF90 7003 9164 9164 D164 D164 3038 0BAA"            /* p.dddd08. */
	$"D154 DF54 2850 2F0C 28AB 0010 296B 0014"            /* TT(P/.(..)k.. */
	$"0004 204B 6100 0094 2F28 0006 4457 446F"            /* .. Ka../(..DWDo */
	$"0002 A8A8 4E75 514F 284F 61DA 206B 0082"            /* ..NuQO(Oa k. */
	$"2450 504A 6114 6604 6110 670A 206B 0082"            /* $PPJa.f.a.g. k. */
	$"2850 61C2 7001 504F 4E75 2F12 2F1A 2F0F"            /* (Pap.PONu/././. */
	$"2F3C FFF9 FFF9 A8A9 4267 2F1C 486F 0006"            /* /<Bg/.Ho.. */
	$"A8AD 5317 101F 504F 4E75 0806 0003 6704"            /* S...PONu....g. */
	$"4A2B 0071 4E75 61F2 6706 206B 0082 A023"            /* J+.qNuag. k.# */
	$"4E75 2F01 206E FFEC 3228 0006 C2FC 0008"            /* Nu/. n2(.... */
	$"B070 1008 6706 5141 6CF6 7200 41F0 100A"            /* p..g.QAlr.A.. */
	$"2008 221F 205F 2F00 4ED0 5448 4A68 0004"            /*  .". _/.NTHJh.. */
	$"6A04 2050 2050 4E75 4A2B 006E 6700 017A"            /* j. P PNuJ+.ng..z */
	$"0C43 0004 6C00 01A4 A89E 6100 017E 4A2E"            /* .C..l..a..~J. */
	$"FFDE 6710 303C 0001 61A8 AA14 303C 0004"            /* g.0<..a.0<.. */
	$"61A0 AA15 4878 09FA 2F17 0806 0000 670A"            /* a.Hx/.....g. */
	$"4A2B 006F 6604 2F17 A8A3 A8A1 2F38 0A02"            /* J+.of././8.. */
	$"A8A9 0806 0001 6704 6100 0248 0806 0000"            /* ....g.a..H.... */
	$"6708 4A2B 006F 6700 00DA 4A2E FFDE 6714"            /* g.J+.og..J.g. */
	$"303C 0004 6100 FF5C AA15 303C 0003 6100"            /* 0<..a.\.0<..a. */
	$"FF52 AA14 0806 0001 6708 3F38 0A00 31C3"            /* R.....g.?8..1 */
	$"0A00 2038 09FA 0280 0007 0007 2855 2940"            /* .. 8.....(U)@ */
	$"FF58 4878 09FA 0806 0001 6706 487A 00E4"            /* XHx....g.Hz. */
	$"6004 487A 00D6 A8A5 0806 0001 6738 6100"            /* `.Hz.֨....g8a. */
	$"FEFA 670C 3017 0440 001A B044 6C02 3800"            /* g.0..@..Dl.8. */
	$"31DF 0A00 31C4 09FC 4878 09FA 487A 00AC"            /* 1..1HxHz. */
	$"3038 09FA 4840 3038 0A00 0280 0007 0007"            /* 08H@08....... */
	$"2940 FF58 A8A5 42AC FF58 4A2B 0070 671A"            /* )@XBXJ+.pg. */
	$"0806 0001 6606 6100 0118 600E 6100 008C"            /* ....f.a...`.a.. */
	$"43FA 02A8 7000 6100 00D8 6100 FE9E 6722"            /* C.p.a..a.g" */
	$"6178 0806 0001 6606 6100 011E 6014 3038"            /* ax....f.a...`.08 */
	$"0A00 3838 09FC 9044 43FA 0240 7000 6100"            /* ..88DC.@p.a. */
	$"0092 2838 0A02 4A2E FFDE 6710 7001 6100"            /* .(8..J.g.p.a. */
	$"FE82 AA14 7000 6100 FE7A AA15 206B 0072"            /* .p.a.z. k.r */
	$"2050 49E8 0002 996C 0004 996C 0006 2F0C"            /*  PI..l..l../. */
	$"A8A1 6100 0462 286B 0072 2854 49EC 0002"            /* a..b(k.r(TI.. */
	$"D96C 0004 D96C 0006 4E75 00AA 00AA 00AA"            /* l..l..Nu... */
	$"00AA 0055 0055 0055 0055 41F8 09FA 226B"            /* ..U.U.U.UA"k */
	$"0072 2251 5449 2099 2151 0004 3010 D047"            /* .r"QTI !Q..0.G */
	$"3140 0004 5368 0006 4E75 6724 6100 FDFC"            /* 1@..Sh..Nug$a. */
	$"6756 0806 0001 6602 6078 61CE 43FA 01CC"            /* gV....f.`xaC. */
	$"7002 41F8 09FA 3168 0006 0002 72EA 6016"            /* p.A1h....r`. */
	$"0806 0001 6602 6032 61B0 43FA 01EE 7002"            /* ....f.`2aC.p. */
	$"41F8 09FA 7206 D368 0002 322E FFFE 08C1"            /* Ar.h..2.. */
	$"0000 D350 5350 2098 7210 D358 D350 223C"            /* ..PSP r.XP"< */
	$"0010 0010 6100 03F6 4E75 610A 6160 4E75"            /* ....a..Nua.a`Nu */
	$"6104 6164 4E75 6100 FF72 616A 41F8 09FA"            /* a.adNua.rajA */
	$"5C68 0002 3428 0002 0642 0009 3142 0006"            /* \h..4(...B.1B.. */
	$"4E75 611E 6138 4E75 6118 613C 6114 6146"            /* Nua.a8Nua.a<a.aF */
	$"41F8 09FA 5768 0006 5768 0004 2F08 A8A1"            /* AWh..Wh../. */
	$"4E75 6100 FF36 612E 41F8 09FA 5D68 0006"            /* Nua.6a.A]h.. */
	$"3428 0006 0442 0009 3142 0002 4E75 6116"            /* 4(...B.1B..Nua. */
	$"4878 09FA A8A4 4E75 4878 09FA 2F17 A8A3"            /* HxNuHx/. */
	$"6104 A8A1 4E75 4878 09FA 2F38 0A02 A8A9"            /* a.NuHx/8.. */
	$"4E75 3838 0A00 3604 9678 09FC 966B 008A"            /* Nu88..6.xk. */
	$"E243 7002 4A2B 0070 6702 7020 B640 6C02"            /* Cp.J+.pg.p @l. */
	$"3600 D678 09FC 3F04 6100 FCF0 6704 0444"            /* 6.x?.a.g..D */
	$"0020 31C4 0A00 A8DA 4878 09FA A8A1 2F2E"            /* . 1..Hx/. */
	$"FFDA A8DB 2055 2050 2F28 001C 2F2E FFDA"            /* ڨ U P/(../. */
	$"2F17 A8E4 31DF 0A00 3F03 3038 09FA D06E"            /* /.1..?.08n */
	$"FFF8 5240 3F00 A893 4A2E FFDE 671A 303C"            /* R@?.J.g.0< */
	$"0004 6100 FCBE AA15 303C 0002 6100 FCB4"            /* ..a..0<..a. */
	$"AA14 3F3C 0001 A889 3003 5D43 D06B 008A"            /* .?<..0.]Ck. */
	$"B044 6C02 3800 5C44 3F38 09FC 31C3 09FC"            /* Dl.8.\D?81 */
	$"3F38 0A00 31C4 0A00 4878 09FA A8A3 31DF"            /* ?8..1..Hx1 */
	$"0A00 31DF 09FC 2F2E FFF4 A87A 2F2E FFDA"            /* ..1/.z/. */
	$"A879 206B 0086 A029 2F10 A884 206B 0086"            /* y k.)/. k. */
	$"A02A 2F2E FFF4 A879 4E75 5555 0000 3FF9"            /* *..yNuUU..? */
	$"2088 2089 2088 2089 2088 3F89 2008 2009"            /*      ? .  */
	$"2008 3FF9 0000 5555 0000 0000 0000 0000"            /*  .?..UU........ */
	$"0180 09A0 05C0 0080 1CF0 1F80 0540 0920"            /* .Ơ.....@  */
	$"0100 0000 0000 0000 0000 AAAA 0000 9FFC"            /* ............ */
	$"1004 9004 1004 9004 1004 9004 1004 9004"            /* ............ */
	$"1004 9FFC 0000 AAAA 0000 0000 0000 0000"            /* ............ */
	$"0080 0490 02A0 0000 0E38 0000 02A0 0490"            /* ......8.... */
	$"0080 0000 0000 0000 0000 4267 2F03 2F2B"            /* .........Bg/./+ */
	$"0076 A8E8 4A1F 6734 0806 0002 6628 4A2B"            /* .vJ.g4....f(J+ */
	$"006F 6722 206B 0076 2050 2028 0006 780E"            /* .og" k.v P (..x. */
	$"9044 B640 6D10 4840 9044 4843 B640 6D06"            /* D@m.H@DHC@m. */
	$"7003 6000 00B6 7001 6000 00B0 6100 FD5C"            /* p.`..p.`..a.\ */
	$"4267 2F03 2F08 A8AD 4A1F 6700 00A2 0806"            /* Bg/./.J.g.... */
	$"0000 6708 4A2B 006F 6700 008E 2003 4840"            /* ..g.J+.og.. .H@ */
	$"204B 6100 FB96 D068 0006 906B 0010 D06E"            /*  Ka.h..k..n */
	$"FFFE 6A74 0640 000B 6B6E 4A2B 0070 672A"            /* jt.@..knJ+.pg* */
	$"2003 D068 0008 906B 0012 0C40 0012 6E1A"            /*  .h..k...@..n. */
	$"0806 0001 660C 0C40 0010 6E0E 5D40 6B48"            /* ....f..@..n.]@kH */
	$"6004 5140 6B42 7004 6040 6100 FB0E 6738"            /* `.Q@kBp.`@a..g8 */
	$"3003 204B 6100 FB44 D068 0008 906B 0016"            /* 0. Ka.Dh..k.. */
	$"0C40 FFED 6D22 0806 0001 6608 0C40 FFEF"            /* .@m"....f..@ */
	$"6D16 6006 0C40 FFF7 6E0E 6100 FA9A 6704"            /* m.`..@n.a.g. */
	$"7006 6006 7005 6002 7002 2D40 0014 4E75"            /* p.`.p.`.p.-@..Nu */
	$"6100 FAC8 6704 6100 FA7E 41F8 09FA 2F08"            /* a.g.a.~A/. */
	$"20EB 0010 20AB 0014 204B 6100 FAEE 2F28"            /*  .. .. Ka./( */
	$"0006 4457 446F 0002 A8A8 2F2B 0076 4878"            /* ..DWDo../+.vHx */
	$"09FA A8DF 283C 0002 0002 2F2B 0072 4878"            /* (<..../+.rHx */
	$"09FA 2F17 2F38 0A06 A8A9 3007 5340 9178"            /* /./8..0.S@x */
	$"09FA E28C A8DF 4A2E FFDE 6710 D9B8 09FE"            /* ⌨J.g.ٸ */
	$"2F2B 0072 4878 09FA A8DF 601E 2F2E FFF4"            /* /+.rHx`./. */
	$"4878 09FA 2F17 2F04 A8A8 A8DF 2F2B 0072"            /* Hx/././+.r */
	$"2F2E FFF4 2F2B 0072 A8E5 4E75 2F03 2F38"            /* /./+.rNu/./8 */
	$"0A06 A8A9 2643 2F13 3007 5340 9153 2F03"            /* ..&C/.0.S@S/. */
	$"A8A1 3F2B 0002 3F2B 0004 0457 0010 2F17"            /* ?+..?+...W../. */
	$"A893 3F6B 0006 0002 A891 2697 2F17 A893"            /* ?k....&/. */
	$"3F6B 0006 0002 A891 3F2B 0006 0457 0010"            /* ?k....?+...W.. */
	$"3F13 2F17 A893 3EAB 0004 A891 2F03 2F38"            /* ?./.>.././8 */
	$"0A02 A8A9 4E75 2F04 A89B 302C 0006 3F00"            /* ..Nu/.0,..?. */
	$"3F14 3F00 302C 0004 3F00 3F2C 0002 3F00"            /* ?.?.0,..?.?,..?. */
	$"4A2E FFDE 660C 4A45 6708 D96F 0002 D96F"            /* J.f.JEg.o..o */
	$"0008 A893 A891 A891 A89E 4E75 41F8 0A0E"            /* ..NuA.. */
	$"20C9 30FC 0002 4298 2081 4878 0A0E 2057"            /*  0..B Hx.. W */
	$"2255 2251 4869 0002 4868 0006 4878 09FA"            /* "U"QHi..Hh..Hx */
	$"3F00 42A7 A8EC 4E75 2055 2F10 2F0B A873"            /* ?.BNu U/./.s */
	$"4A2E FFDE 671E 5D4F 2F0F AA19 5D4F 2F0F"            /* J.g.]O/..]O/. */
	$"AA1A 4240 6100 F97C AA15 303C 0001 6100"            /* .B@a.|.0<..a. */
	$"F972 AA14 49F8 09FA 296B 0014 0004 28AC"            /* r.I)k....( */
	$"0004 0454 000F 046C 000F 0002 4A2B 006F"            /* ...T...l....J+.o */
	$"6748 43FA 004E 7000 223C 0010 0010 6100"            /* gHC.Np."<....a. */
	$"FF7C 3F2C 0002 3F17 3F2B 0010 A893 3F2B"            /* |?,..?.?+..?+ */
	$"0014 A891 3F2B 0012 3F14 A893 3F2B 0016"            /* ..?+..?.?+.. */
	$"3F14 A891 4A2E FFDE 670C 2F0F AA15 5C4F"            /* ?.J.g./..\O */
	$"2F0F AA14 5C4F A873 4E75 4878 09FA A8A3"            /* /..\OsNuHx */
	$"60C0 0000 0000 0000 1FE0 1020 103E 1022"            /* `........ .>." */
	$"1022 1022 1022 1FE2 0402 0402 0402 07FE"            /* ."."."........ */
	$"0000"                                               /* .. */
};
