/*
	File:		DrawEditorGlobals.h

	Contains:	DrawEditorGlobals globals

	Written by:	Dave Stafford

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/


#ifndef _DRAWEDITORGLOBALS_
#define _DRAWEDITORGLOBALS_

// -- DrawEditor --

#ifndef _PALETTE_
#include "Palette.h"
#endif

#ifndef _DRAWEDITORCONSTANTS_
#include "DrawEditorConstants.h"
#endif

// -- OpenDoc --

#ifndef SOM_ODMenuBar_xh
#include <MenuBar.xh>
#endif

#ifndef SOM_Module_OpenDoc_Global_TypesB_defined
#include <ODTypesB.xh>
#endif

// -- OpenDoc Public Utilities --

#ifndef _ITEXT_
#include <IText.h>
#endif

// -- Forward Class declarations --
class CColorPalette;
class CToolPalette;
class CRGBColor;
class ODShape;
class CClipboardPromise;


//==============================================================================
// Globals
//==============================================================================
struct DrawEditorGlobals; // forward

extern	ODUShort			gGlobalsUsageCount;
extern	DrawEditorGlobals*	gGlobals;
	
struct DrawEditorGlobals {
	public:
	DrawEditorGlobals();
	~DrawEditorGlobals() {}
	
	ODMenuBar*		fMenuBar;
	ODFocusSet*		fUIFocusSet;	
	Handle			fThumbnail;
				
	ODTypeToken		fSelectionFocus;
	ODTypeToken		fMenuFocus;
	ODTypeToken		fModalFocus;
	ODTypeToken		fClipboardFocus;
	ODTypeToken		fKeyFocus;
	
	ODTypeToken		fFrameView;
	ODTypeToken		fLargeIconView;
	ODTypeToken		fSmallIconView;
	ODTypeToken		fThumbnailView;
	
	ODTypeToken		fMainPresentation;
	ODTypeToken		fUndefinedPresentation;
	
	ODType			fCurrentTextPartKind;
			
	ODScriptCode	fEditorsScript;
	ODLangCode		fEditorsLanguage;
	
	Handle			fFloatWindowProc;

	// Clipboard
	CClipboardPromise* fClipboardPromise;

	// Palettes
	CColorPalette*	fColorPalette;
	CToolPalette*	fToolPalette;

	ODBoolean		fShowColorPalette;
	ODBoolean		fShowToolPalette;

	// Global constants
	CRGBColor*	 	fBlackColor;
	ODRect	 		fZeroODRect;
	Rect	 		fZeroRect;
	Rect	 		fMinShapeSizeRect;
	ODULong	 		fZeroLong;


// Global flags
	ODBoolean 		fDrawEditorRequestingFocus;
	
};


inline DrawEditorGlobals::DrawEditorGlobals()
{
	fMenuBar				= kODNULL;
	fUIFocusSet				= kODNULL;	
	fThumbnail				= kODNULL;
				
	fSelectionFocus			= kODNullTypeToken;
	fMenuFocus				= kODNullTypeToken;
	fModalFocus				= kODNullTypeToken;
	fKeyFocus				= kODNullTypeToken;
	fClipboardFocus			= kODNullTypeToken;
	
	fFrameView				= kODNullTypeToken;
	fLargeIconView			= kODNullTypeToken;
	fSmallIconView			= kODNullTypeToken;
	fThumbnailView			= kODNullTypeToken;
	
	fMainPresentation		= kODNullTypeToken;
	fUndefinedPresentation	= kODNullTypeToken;
			
	fCurrentTextPartKind	= kODNullTypeToken;
	fFloatWindowProc		= kODNULL;
	fColorPalette			= kODNULL;
	fToolPalette			= kODNULL;
	
	fClipboardPromise		= kODNULL;
	
	fEditorsScript			= 0;
	fEditorsLanguage		= 0;
	
	fBlackColor				= kODNULL;
	
	// Initialize global constants
	fMinShapeSizeRect.top 	= 0;
	fMinShapeSizeRect.left 	= 0;
	fMinShapeSizeRect.bottom = kMinShapeHeight;
	fMinShapeSizeRect.right	= kMinShapeWidth;
	
	fZeroRect.top 			= 0;
	fZeroRect.left 			= 0;
	fZeroRect.bottom	 	= 0;
	fZeroRect.right 		= 0;
	
	fZeroODRect.top 		= 0;
	fZeroODRect.left 		= 0;
	fZeroODRect.bottom 		= 0;
	fZeroODRect.right 		= 0;
	
	fZeroLong 				= 0L;
	
	fShowToolPalette 		= kODFalse;
	fShowColorPalette 		= kODFalse;
	fDrawEditorRequestingFocus = kODFalse;
}
			
#endif