/*
	File:		Palette.h

	Contains:	Palette Classes Definition

	Written by:	Dave Stafford
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

// -- Compiler/Preprocessor Switches --

#ifndef _PROMISE_
#define _PROMISE_

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

#ifndef _DRAWEDITORDEF_
#include "DrawEditorDef.h"
#endif

//=============================================================================
// Forward Declarations
//=============================================================================
class DrawEditor;
class CSelection;
class ODFrame;
class COrderedList;
class CDrawContent;

//=============================================================================
// Constants
//=============================================================================

//=============================================================================
// CPromise
//=============================================================================
class CPromise
{
public:
	
	// -- Init --
	CPromise(DrawEditor* editor, 
					ODFrame* sourceFrame, 
					COrderedList* shapeList,
					ODCloneKind kind);

	CPromise(DrawEditor* editor, 
					ODFrame* sourceFrame, 
					CSelection* selection, 
					ODCloneKind kind);
	~CPromise();

public:
	virtual void 	FulfillPromise(Environment* ev, ODStorageUnit* storage);
	void 			FulfillPromise(Environment* ev, ODStorageUnitView* view);
	void 			Promise(Environment *ev, ODStorageUnit* storageUnit);
	ODBoolean		OKToDelete()
	{
		return (fCloneKind != kODCloneToLink);  
	}
	COrderedList* GetShapeList();
	CDrawContent* GetPromiseContent();

	

//----------------------------------------------------------------------------------------
// Data Members
//
//
//
//
protected:
	ODFrame*			fSourceFrame;
	DrawEditor*			fDrawEditor;
	
	ODCloneKind			fCloneKind;
	CDrawContent*		fPromiseContent;
	
};



//=============================================================================
// CClipboardPromise
//=============================================================================
class CClipboardPromise: public CPromise
{
public:
	
	// -- Init --
	CClipboardPromise(DrawEditor* editor, 
					ODFrame* sourceFrame, 
					CSelection* selection, 
					ODCloneKind kind,
					ODUpdateID updateID);
	~CClipboardPromise();

public:

	virtual void 	FulfillPromise(Environment* ev, ODStorageUnit* storage);
	virtual void 	Promise(Environment *ev, ODStorageUnit* storageUnit);
	virtual void 	ShapesPromisedToClipboard(Environment *ev, ODBoolean promised);
	ODUpdateID 		GetUpdateID();

private:
	// Used to test clipboard for our promises.
	ODUpdateID	fUpdateID;  


};





#endif