/*
	File:		ListItem.h

	Contains:	List Item Classes Definition

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _LISTITEM_
#define _LISTITEM_

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif


class ODFacet;
class ODFrame;
class ODStorageUnit;
class ODIText;
class CList;


//=============================================================================
// CProxy
//=============================================================================

class CEmbeddedFrameProxy
{
	public:
	CEmbeddedFrameProxy();
	virtual ~CEmbeddedFrameProxy();
	
	void		InitEmbeddedFrameProxy(Environment* ev, ODFrame* frame);
	void		Release(Environment* ev);
	ODFrame*	GetFrame();
	void		SetFrame(Environment* ev, ODFrame* frame);
	ODFrame*	GetContainingFrame();
	void		SetContainingFrame(Environment* ev, ODFrame* frame);
	
	private:
	ODBoolean	fInited;	
	ODFrame*	fFrame;
	ODFrame*	fContainingFrame;
};

//=============================================================================
// CListItem
//=============================================================================

class CListItem
{
	public:
	
	// --- Init ---
	CListItem();
	void		InitListItem();
		
	// --- Ref Counting ---
	void		Acquire();
	void		Release(Environment* ev);
	
	// --- Name ---
	void		SetName(Environment* ev, ODIText* name);
	ODIText*	GetName(Environment* ev);
	void		RetrieveName(Environment* ev);
	
	// --- Misc ---
	void		SetListPosition(Environment* ev, ODUShort pos);
	ODUShort	GetListPosition();
	ODPart*		AcquirePart(Environment* ev);
	void		DoPartInfo(Environment* ev, ODFrame* containingFrame);

	// --- Imaging ---
	void		Check(Environment* ev);
	void		Draw(Environment* ev, ODFacet* facet);
	void		Invalidate(Environment* ev);

	// --- Selection ---
	ODBoolean	HitTest(Environment* ev, ODFrame* frame, Point pt);
	ODBoolean	IsSelected() const;
	void		SetSelected(Environment *ev, ODBoolean selected);
	void		ActivationChanged(Environment* ev, ODID windowID, ODBoolean actRes);
	RgnHandle	GetSelectedRegion(Environment* ev);
	
	// --- Embedding ---
	void		AddFrame(Environment* ev, ODFrame* displayFrame);
	void		RemoveFrame(Environment* ev, ODFrame* displayFrame);
	void		FrameClosed(Environment* ev, ODFrame* containingFrame);
	void		FrameRemoved(Environment* ev, ODFrame* containingFrame);
	void		AttachSourceFrame(Environment* ev, ODFrame* containingFrame,
								ODFrame* containingSourceFrame);
	void		AddFacet(Environment* ev, ODFacet* containingFacet);
	void		RemoveFacet(Environment* ev, ODFacet* containingFacet);
	void		Reattach(Environment* ev);
	void		Detach(Environment* ev);
	
	// --- Storage ---
	void 		Read(Environment* ev, ODStorageUnit* storage);
	void 		Write(Environment* ev, ODStorageUnit* storage, ODDraft* draft,
								ODDraftKey key);
	void		ClonePart(Environment* ev, ODStorageUnit* su, ODDraftKey key);
		
	protected:
	
	// --- Destruction ---
	virtual 	~CListItem();
	void		ReleaseAll(Environment* ev);
	
	// --- Imaging ---
	void		CalcNameRect();
	void		FocusAndDraw(Environment *ev, ODFacet* embFacet,
								ODBoolean erase = kODFalse);
	void		DrawAll(Environment *ev, ODBoolean erase = kODFalse);
	void		DrawName(ODBoolean active, ODBoolean erase = kODFalse);
	
	private:
	ODBoolean		fInited;
	ODUShort		fRefCount;
	ODPart*			fPart;
	ODIText* 		fName;
	Rect			fNameRect;
	ODBoolean		fSelected;
	ODUShort		fPosition;
	Point			fItemOffset;
	CList*			fFrameProxies;
	RgnHandle		fInvalRgn;
};

#endif