/*
	File:		Panel.h

	Contains:	Panel Classes Definition

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _PANEL_
#define _PANEL_

// -- PanelEditor Includes --

#ifndef _FRAMEPE_
#include "FramePE.h"
#endif

#ifndef _PANELEDITORDEF_
#include "PanelEditorDef.h"
#endif

#ifndef _PANELEDITORGLOBALS_
#include "PanelEditorGlobals.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif


class CListItem;
class ODPart;

//=============================================================================
// CPanel
//=============================================================================

class CPanel : public CFrame
{
	public:
	
	// --- Init ---
	CPanel();
	~CPanel();
	void	InitPanel();
	
	// --- Layout ---
	void	CreateFrames(Environment* ev);
	void	RemoveAll(Environment* ev);
	void	AddFrame(Environment* ev, ODFrame* containingFrame);
	ODFrame* GetContainedFrame(Environment* ev, ODFrame* containingFrame);

	// --- Imaging ---
	void	AddFacet(Environment* ev, ODFacet* containgingFacet);
	void	Draw(Environment* ev, ODFacet* containgingFacet);
	void	ChangeDisplayItem(Environment* ev, CListItem* item);
	
	protected:
	
	// --- Getters ---
	ODRect		GetFrameBounds(Environment* ev, ODFrame* containingFrame);
	Point		GetFrameOffset();
	ODTypeToken	GetPresentation();
	ODBoolean	MakeSubframe();
	ODObjectType	GetFrameType();
	
	private:
	ODBoolean		fHasFrames;

};

//-----------------------------------------------------------------------------
// Inlines
//-----------------------------------------------------------------------------

inline ODTypeToken CPanel::GetPresentation()
{ return gGlobals->fUndefinedPresentation; }


inline ODBoolean CPanel::MakeSubframe()
{ return kODFalse; }


inline Point CPanel::GetFrameOffset()
{ Point offset = {kPanelTopEdge+1,kPanelLeftEdge+1}; return offset; }


inline ODObjectType CPanel::GetFrameType()
{ return kODNonPersistentFrameObject; }


#endif