/*
	File:		ScrollingList.h

	Contains:	Scrollling lits Classes Definition

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _SCROLLINGLIST_
#define _SCROLLINGLIST_

// -- PanelEditor Includes --

#ifndef _FRAMEPE_
#include "FramePE.h"
#endif

#ifndef _PANELEDITOR_
#include "PanelEditor.h"
#endif

#ifndef _SAMPLECOLLECTIONS_
#include "SampleCollections.h"
#endif

#ifndef _PANELEDITORDEF_
#include "PanelEditorDef.h"
#endif

#ifndef _PANELEDITORGLOBALS_
#include "PanelEditorGlobals.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

// -- OpenDoc Utilities --

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif


class CListItem;
class ODPart;

//=============================================================================
// CScrollingList
//=============================================================================

class CScrollingList : public CFrame
{
	public:
	
	// --- Init ---
	
	CScrollingList();
	~CScrollingList();
	void	InitScrollingList(Environment* ev, PanelEditor* pe);
	
	// --- Getters ---
	
	ODFrame*	GetFrame(Environment* ev, ODFrame* hintFrame);
	void		SetPartDisplayed(Environment* ev, ODPart* part);
	
	// --- Storage ---
	void	Internalize(Environment* ev, ODStorageUnit* storageUnit);
	void	Externalize(Environment* ev, ODStorageUnit* storageUnit,
							ODDraft* draft, ODDraftKey key);
	
	// --- Layout ---
	void	PrepareFrame(Environment* ev, ODFrame* frame);
	void	CleanupFrame(Environment* ev, ODFrame* frame, ODBoolean frameRemoved);
	void	SourceFrameAttached(Environment* ev, ODFrame* frame,
								ODFrame* sourceFrame);

	// --- Imaging ---
	void	Draw(Environment* ev, ODFacet* facet);
	void	PrepareFacet(Environment* ev, ODFacet* facet);
	void	CleanupFacet(Environment* ev, ODFacet* facet);
	void	FacetChanged(Environment* ev, ODFacet* facet);
	void	ClipItems(Environment* ev);
	void	AdjustScrollbar(Environment* ev/*, ODFrame* frame*/);
	void	AdjustItemPositions(Environment* ev);
	
	// --- Embedding ---
	ODBoolean	HitTest(Environment* ev, ODFacet* facet,
						ODFacet* embFacet, Point pt, CListItem** item);
	CListItem*	EmbedPart(Environment* ev, ODPart* part);
	
	protected:
	
	// --- Getters ---
	ODRect		GetFrameBounds(Environment* ev, ODFrame* containingFrame);
	Point		GetFrameOffset();
	ODTypeToken	GetPresentation();
	ODBoolean	MakeSubframe();
	COrderedList*	GetContent();
	ODObjectType	GetFrameType();
	
	// --- Embedding ---
	void		EmbedItemsInFrame(Environment* ev, ODFrame* frame);
	
	private:
	PanelEditor*	fPE;

};


//--------------------------------------------------------------------------
// Inlines
//--------------------------------------------------------------------------

inline void CScrollingList::SetPartDisplayed(Environment* ev, ODPart* part)
{
	// only do this once.
	if ( !fPartDisplayed )
	{
		ODAcquireObject(ev,part);
		fPartDisplayed = part;
	}
}


inline ODTypeToken CScrollingList::GetPresentation()
{ return gGlobals->fListPresentation; }


inline ODBoolean CScrollingList::MakeSubframe()
{ return kODTrue; }


inline Point CScrollingList::GetFrameOffset()
{ Point offset = {kListTopEdge,kListTopEdge}; return offset; }


inline COrderedList* CScrollingList::GetContent()
{ return fPE->GetEmbeddedItems(); }


inline ODObjectType CScrollingList::GetFrameType()
{ return kODNonPersistentFrameObject; }


#endif