/*
	File:		SoundEditorDef.h

	Contains:	Common definitions used by SoundEditor.cpp & .r files

	Written by:	Steve Smith and Andrey Dolgachev
 
	Copyright:	 1994-95 by Apple Computer, Inc., all rights reserved.

	-------------------------------------------------------------------
	
	Note:		This file will need to be modified for any derivitive
				work based on SamplePart. The areas marked with (CH)
				should be considered when making those changes.
*/

#ifndef _SOUNDEDITORDEF_
#define _SOUNDEDITORDEF_

// -- OpenDoc Includes --

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

//=====================================================================

// Class / Editor ID (CH)
#define kPartClassName					"som_SoundEditor"
#define kSoundEditorID 					"SampleCode::"kPartClassName

// Editor User String (CH)
#define kSoundEditorUserString			"Sound Editor 1.0"

// Kinds (CH)
#define kSoundEditorKind				kODISOPrefix "Apple:Kind:SoundEditor"
#define kSoundDataKind					'snd '
#define kSoundFileKind					'sfil'

// Kind User Strings (CH)
#define kSoundEditorKindUserString		"Sampled sound"
#define kSoundDataKindUserString		"Sampled sound data"
#define kSoundFileKindUserString		"Sampled sound file"

// SoundEditor OSType (CH)
#define kSoundEditorOSType				'SNED'
#define kSoundViewerOSType				'SNVW'
#define kSoundEditorDocumentOSType		'SNDC'
#define kSoundEditorStationeryOSType	'sNDC'

// Sound Editor ISO Strings (CH)
#define	kControlsPresentation			kODISOPrefix "Sound Editor:Presentation:Controls"

#define kSoundEditorInfo				kODISOPrefix "Sound Editor:Display Frame Info"

#define	kPropRecordingQuality			kODISOPrefix "Content:Property:Sound Recording Quality"
#define	kPropSecsRecorded				kODISOPrefix "Content:Property:Recorded Sound Length"
#define	kOSType							kODISOPrefix "Apple:Type:OSType"

#define	kSoundInFocus					"SoundEditor:Sound In"
#define	kSoundOutFocus					"SoundEditor:Sound Out"

// Sound Editor defines
#define kBaseResourceID					20001

// 'nmap' Resource IDs
#define kKindCategoryMapId				kBaseResourceID+1
#define kEditorKindMapId				kBaseResourceID+2
#define kEditorUserStringMapId			kBaseResourceID+3
#define kKindUserStringMapId			kBaseResourceID+4
#define kOldMacOSTypeMapId				kBaseResourceID+6
#define kPlatformEditorKindMapId		kBaseResourceID+7

// Text Items
#define	kMenuStringResID				kBaseResourceID
#define	kAboutTextID       				 1

// Error Messages
#define	kErrorStringResID				kBaseResourceID+1
#define kErrCantInitializePart			 1
#define kErrCantOpenDocWindow			 2
#define kErrCantOpenPartWindow			 3
#define kErrRemoveFrame					 4
#define kErrCantGetClipboard			 5
#define kErrOutOfMemory					 6
#define	kErrCannotInitSndChannel		 7
#define kErrCannotOpenSndDevice			 8
#define	kErrSndChannelNotInited			 9
#define kErrSndDeviceNotOpened			10
#define kErrEditorPlaying				11
#define kErrEditorRecording				12
#define	kErrCannotPlaySound				13
#define	kErrCannotRecordSound			14
#define kErrNoSoundData					15
#define kErrEditorModal					16
#define kErrWindowGone					17
#define kErrExternalizeFailed			18

// Bundles/FREFs
#define kDocumentBundle					kBaseResourceID
#define kEditorBundle					kBaseResourceID+1
#define kViewerBundle					kBaseResourceID+2
#define kDocumentFREF					kBaseResourceID
#define kStationeryFREF					kBaseResourceID+1
#define kEditorFREF						kBaseResourceID+2
#define kViewerFREF						kBaseResourceID+3

// Sound Editor Dialogs & Windows (CH)
#define	kAboutBoxID  					kBaseResourceID
#define	kErrorBoxID  					kBaseResourceID+1
#define kSettingsDialogID				kBaseResourceID+2
#define	kErrStrFieldID					3

#define kDialogFontInfoID				kBaseResourceID+2
#define kGoodControl					kBaseResourceID
#define kBetterControl					kBaseResourceID+1
#define kBestControl					kBaseResourceID+2
#define	kGood							5
#define	kBetter							6
#define	kBest							7
#define kDefaultHighlight				8

// Windows
#define	kMacWindowTitleBarHeight		20
#define	kALittleNudge					4
#define kMinVertVisPortion				10
#define kMinHorzVisPortion				16
#define kWindowOpening					1
#define kWindowClosing					0

// Sound Editor icons
#define	kLargeIcons						1
#define	kSmallIcons						2
#define	kDocumentIcons					kBaseResourceID
#define	kStationeryIcons				kBaseResourceID+1
#define	kEditorIcons					kBaseResourceID+2
#define	kViewerIcons					kBaseResourceID+3

// Display Frames
#define kFrameRemoved					1
#define kFrameClosed					0

// Geometry
#define kMinImagingResolution			72 // dpi

// Sound Editor Constants (CH)
#define	kControlsWidth					241
#define	kControlsHeight					88
#define kSoundEditorIdle				20 // ticks

// Button icons (CH)
#define	kRecordButtonIcon				kBaseResourceID+9
#define	kStopButtonIcon					kRecordButtonIcon+1
#define	kPauseButtonIcon				kRecordButtonIcon+2
#define	kPlayButtonIcon					kRecordButtonIcon+3

// Sound Editor Icons & Pictures (CH)
#define kEditorIconPicture				kEditorIcons
#define	kThumbnailPICT					kBaseResourceID+1
#define	kControlsPICT					kBaseResourceID+3

#define	kSpeaker0PICT					kBaseResourceID+12
#define	kSpeaker1PICT					kSpeaker0PICT+1
#define	kSpeaker2PICT					kSpeaker0PICT+2
#define	kSpeaker3PICT					kSpeaker0PICT+3
#define	kSpeaker4PICT					kSpeaker0PICT+4
#define	kSpeaker5PICT					kSpeaker0PICT+5

#define kNormalButtonPICT				kBaseResourceID+19
#define kPressedButtonPICT				kNormalButtonPICT+1
#define kOnButtonPICT					kNormalButtonPICT+2
#define kDimButtonPICT					kNormalButtonPICT+3

// Sound Editor PixPats (CH)
#define kRecordOnPAT					kBaseResourceID
#define kRecordOffPAT					kBaseResourceID+1
#define kPlayingPAT						kBaseResourceID+2
#define kInvalidPAT						kBaseResourceID+3

// Sound Editor (menu) commands (CH)
#define kBaseMenuID						kBaseResourceID
#define kNumMenus						1
#define	kSoundMenuID					kBaseMenuID

#define	kPlayCommand					kSoundMenuID+1
#define	kPauseCommand					kSoundMenuID+2
#define	kStopCommand					kSoundMenuID+3
#define	kRecordCommand					kSoundMenuID+4
#define	kSettingsCommand				kSoundMenuID+5

// Sound Editor AETE defines (CH)

#define	kSoundEditorAEID				'Soud'
#define kSoundEditorAEClass				'soud'

#define	cEditorClass					'cEdt'

#define	kRecordSound					'recd'
#define	kStopSound						'stpp'
#define	kPlaySound						'play'
#define	kPauseSound						'paws'
#define	kResumeSound					'resm'

#define	pState							'psta'
#define	pQuality						'pqty'
#define pCurrentTime					'crTm'
#define	pMaxTime						'mxTm'
#define	pSndQuality						'sqty'

#define	enumQuality						'qwlt'
#define kUnknownQuality					'unkn'
#define kGoodQuality					'good'
#define kBetterQuality					'betr'
#define	kBestQuality					'best'

#define	enumState						'stat'
#define kStopState						'stSt'
#define	kRecordState					'rcSt'
#define kPlayState						'plSt'
#define	kPauseRecordState				'prSt'
#define	kPausePlayState					'ppSt'

#endif

