/*
	File:		som_SoundEditorSI.cpp

	Contains:	Sound Editor Semantic Interface extension

	Written by:	Andrey Dolgachev
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

#define VARIABLE_MACROS
#define SoundEditorSI_Class_Source

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (eg. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- SoundEditorSI Includes --

#ifndef SOM_SoundEditorSI_xih
#include "som_SoundEditorSI.xih"
#endif

// -- SoundEditor Includes --

#ifndef _SOUNDEDITOR_
#include "SoundEditor.h"
#endif

#ifndef _SOUNDEDITORDEF_
#include "SoundEditorDef.h"
#endif

#ifndef _SOUNDEDITOROBJ_
#include "SoundEditorObj.h"
#endif

// -- OpenDoc Includes --

#ifndef SOM_ODMessageInterface_xh
#include "MssgIntf.xh"
#endif

#ifndef SOM_ODNameResolver_xh
#include <NamRslvr.xh>
#endif

#ifndef SOM_ODAppleEvent_xh
#include <ODAplEvt.xh>
#endif

#ifndef SOM_ODOSLToken_xh
#include <ODOSLTkn.xh>
#endif

#ifndef SOM_Module_OpenDoc_ODRegistry_defined
#include <ODRgstry.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _FOCUSLIB_
#include <FocusLib.h>
#endif

#ifndef _ODDESUTL_
#include <ODDesUtl.h>
#endif

#ifndef _ODMEMORY_
#include <ODMemory.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

// -- Toolbox Includes --

#ifndef __AEOBJECTS__
#include <AEObjects.h>
#endif

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __ASREGISTRY__
#include <ASRegistry.h>
#endif


#pragma segment SoundEditorSI

//------------------------------------------------------------------------------
// Macros
//------------------------------------------------------------------------------

#define NULLDESC {typeNull, kODNULL}

#define SETBYTESOFHANDLE(h,typ,data,skipbytes)							\
	*((typ*)((*(h))+(skipbytes))) = (data)								\

#define GETBYTESOFHANDLE(h,typ,skipbytes)								\
	*((typ*)((*(h))+(skipbytes)))										\

#define SETFIRSTBYTESOFHANDLE(h,typ,data)								\
	SETBYTESOFHANDLE(h,typ,data,0)										\

#define FIRSTBYTESFROMHANDLE(h, typ)									\
		GETBYTESOFHANDLE(h,typ,0)										\

ODError HandleSetData( ODPart* part, ODAppleEvent* message, 
							ODAppleEvent* reply, ODPart* editor);
ODError HandleGetData( ODPart* part, ODAppleEvent* message, 
							ODAppleEvent* reply, ODPart* editor);
void GetDirectParam( ODSession* session, AppleEvent* message,
							AEDesc* evtDp );

//------------------------------------------------------------------------------
// GetSLongAttr
//------------------------------------------------------------------------------
static ODSLong GetSLongAttr(AppleEvent* ae, AEKeyword keyword) // Copied from SEUtils.cpp
{
	ODSLong		value;
	DescType	actualType;
	Size		sizeOfBuffer = sizeof(value);
	Size		actualSize;

	THROW_IF_ERROR(AEGetAttributePtr(ae, keyword, typeWildCard, &actualType,
									 (Ptr) &value, sizeOfBuffer, &actualSize));

	if (actualSize != sizeOfBuffer)
		THROW(kODErrOutOfMemory);
		
	return value;
}

//------------------------------------------------------------------------------
// RegisterRealPart
//------------------------------------------------------------------------------
SOM_Scope void  SOMLINK SoundEditorSI__RegisterRealPart(SoundEditorSI *somSelf, Environment *ev,
		ODPtr realPart)
{
    SoundEditorSIData *somThis = SoundEditorSIGetData(somSelf);
    SoundEditorSIMethodDebug("SoundEditorSI","SoundEditorSI__RegisterRealPart");

	_fRealPart = (SoundEditor*) realPart;
}

//------------------------------------------------------------------------------
// somInit
//------------------------------------------------------------------------------
SOM_Scope void  SOMLINK SoundEditorSI__somInit(SoundEditorSI *somSelf)
{
//	SoundEditorSIData *somThis = SoundEditorSIGetData(somSelf);
    SoundEditorSIMethodDebug("SoundEditorSI","SoundEditorSI__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically.
	// Thus, there is no need to call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
}

//------------------------------------------------------------------------------
// somUninit
//------------------------------------------------------------------------------
SOM_Scope void  SOMLINK SoundEditorSI__somUninit(SoundEditorSI *somSelf)
{
//	SoundEditorSIData *somThis = SoundEditorSIGetData(somSelf);
    SoundEditorSIMethodDebug("SoundEditorSI","SoundEditorSI__somUninit");
}

//------------------------------------------------------------------------------
// CallEventHandler
//------------------------------------------------------------------------------
SOM_Scope void  SOMLINK SoundEditorSI__CallEventHandler(SoundEditorSI *somSelf, Environment *ev,
		ODPart* 		thePart,
		ODAppleEvent* 	theODAppleEvent,
		ODAppleEvent* 	reply )
{
    SoundEditorSIData *somThis = SoundEditorSIGetData(somSelf);
    SoundEditorSIMethodDebug("SoundEditorSI","SoundEditorSI__CallEventHandler");

	ODPart*			editor = (ODPart*) somSelf->GetBase(ev);
	
	AppleEvent		aeEvent;
	AppleEvent		aeReply;

	AEEventClass 	eventClass;
	AEEventID		eventID;

	TRY								

		THROW_IF_ERROR( ODDescToAEDesc(theODAppleEvent, &aeEvent) );
		THROW_IF_ERROR( ODDescToAEDesc(reply, &aeReply) );
				
		// Get the event class and ID
		eventClass = (AEEventClass) GetSLongAttr(&aeEvent, keyEventClassAttr);
		eventID = (AEEventID) GetSLongAttr(&aeEvent, keyEventIDAttr);

		// Handle the apple event based on the eventID
		switch(eventID)
		{
			// Handle the Get and Set events
			case kAESetData:
				HandleSetData(thePart, theODAppleEvent, reply, editor);
				break;
			
			case kAEGetData:
				HandleGetData(thePart, theODAppleEvent, reply, editor);
				break;
			
			// Handle the Sound Editor specific editors
			case kRecordSound:
				if( !_fRealPart->SoundRecording() || _fRealPart->SoundPaused() )
					_fRealPart->HandleSoundEvent(ev, kRecordCommand);
				break;
						
			case kStopSound:
				if( _fRealPart->SoundPlaying() || _fRealPart->SoundRecording() )
					_fRealPart->HandleSoundEvent(ev, kStopCommand);
				break;
			
			case kPlaySound:
				if( ( !_fRealPart->SoundPlaying() || _fRealPart->SoundPaused() ) 
				     	&& !_fRealPart->SoundRecording() )
					_fRealPart->HandleSoundEvent(ev, kPlayCommand);
				break;
			
			case kPauseSound:
				if( ( _fRealPart->SoundPlaying() || _fRealPart->SoundRecording() )
						&& !_fRealPart->SoundPaused() )
					_fRealPart->HandleSoundEvent(ev, kPauseCommand);
				break;

			case kResumeSound:
				if( ( _fRealPart->SoundPlaying() || _fRealPart->SoundRecording() )
						&& _fRealPart->SoundPaused() )
					_fRealPart->HandleSoundEvent(ev, kPauseCommand);
				break;
			
		}
				
	CATCH_ALL
	ENDTRY
	
	AEDisposeDesc(&aeEvent);
	AEDisposeDesc(&aeReply);
}

//------------------------------------------------------------------------------
// CallObjectAccessor
//------------------------------------------------------------------------------
SOM_Scope void  SOMLINK SoundEditorSI__CallObjectAccessor(
						SoundEditorSI	*somSelf, 	
						Environment 	*ev,
						ODPart* 		thePart,
						ODDescType 		desiredClass,
						ODOSLToken* 	container,
						ODDescType 		containerClass,
						ODDescType 		form,
						ODDesc* 		selectionData,
						ODOSLToken* 	value )
{
    SoundEditorSIData *somThis = SoundEditorSIGetData(somSelf);
    SoundEditorSIMethodDebug("SoundEditorSI","SoundEditorSI__CallObjectAccessor");

	ODSession*			session = ODGetSession(ev, (ODPart*) somSelf->GetBase(ev));
	ODNameResolver* 	resolver = session->GetNameResolver(ev);

	EditorPropAccessor*	editorAccObj;

	AEDesc				realData = NULLDESC;
	AEDesc				tokenDesc = NULLDESC;

	ODVolatile(tokenDesc);
	ODVolatile(realData);
	
	SOM_TRY
		if (form != formPropertyID)
			THROW(errAEEventNotHandled);
	
		THROW_IF_ERROR(ODDescToAEDesc(selectionData, &realData));
	
		DescType propID = **(DescType**) realData.dataHandle;
	
		switch (propID)
		{
			case pState:
			case pQuality:
			case pCurrentTime:
			case pMaxTime:
			case pLength:		
			case pSndQuality:
				if ( !resolver->IsODToken(ev, value) )
					THROW( errAEEventNotHandled /*errAENoSuchObject*/ );
				tokenDesc.descriptorType = cProperty;
				tokenDesc.dataHandle = ODNewHandle(sizeof(editorAccObj));
	
				editorAccObj = new EditorPropAccessor(propID, _fRealPart);
				THROW_IF_NULL(editorAccObj);
				SETFIRSTBYTESOFHANDLE(tokenDesc.dataHandle, EditorPropAccessor*,
									  editorAccObj);
				
				ODDesc* userODDesc = resolver->GetUserToken(ev, value);
				THROW_IF_ERROR( AEDescToODDesc( &tokenDesc, userODDesc ) );
				break;
				
			default:
				THROW(errAEEventNotHandled);
				break;
		}
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	AEDisposeDesc( &tokenDesc );
	AEDisposeDesc( &realData );
}

//------------------------------------------------------------------------------
// Utility functions
//------------------------------------------------------------------------------

void GetDirectParam(ODSession* session, AppleEvent* message,
		AEDesc* evtDp)
{
	Environment* ev = somGetGlobalEnvironment();
	AEDesc localDP;
	THROW_IF_ERROR(AEGetParamDesc(message, keyDirectObject, typeWildCard,
			&localDP));

	ODNameResolver* resolver = session->GetNameResolver(ev);

	ODOSLToken* tmpWrapper = new ODOSLToken();
	THROW_IF_NULL(tmpWrapper);
	tmpWrapper->InitODOSLToken(ev);
	THROW_IF_ERROR( AEDescToODDesc(&localDP, tmpWrapper ) );

	ODBoolean isToken = resolver->IsODToken(ev, tmpWrapper);
	ODDeleteObject( tmpWrapper );
	if ( isToken )
	{
		*evtDp = localDP;
	}
	else
	{
		AEDisposeDesc(&localDP);
		THROW( errAEEventNotHandled );
	}
}	// GetDirectParam()

//------------------------------------------------------------------------------
// HandleSetData
//------------------------------------------------------------------------------

ODError HandleSetData(				ODPart*			part,
									ODAppleEvent*	message,
									ODAppleEvent*	reply,
									ODPart*			editor)
{
	AEDesc theData = NULLDESC;
	AEDesc evtDp = NULLDESC;
	AEDesc theToken = NULLDESC;
	AppleEvent realMessage = NULLDESC;
	AppleEvent realReply = NULLDESC;
	ODError	error = noErr;
	
	Environment*		ev = somGetGlobalEnvironment();
	ODSession*			session =  ODGetSession(ev, editor);
	ODNameResolver* 	resolver = session->GetNameResolver(ev);

	THROW_IF_ERROR( ODDescToAEDesc(message, &realMessage) );
	THROW_IF_ERROR( ODDescToAEDesc(reply, &realReply ));

	ODOSLToken* tmpWrapper = kODNULL;
	TRY
		
		THROW_IF_ERROR(AEGetKeyDesc(&realMessage, keyAEData,
	              typeWildCard, &theData));
	
		GetDirectParam( session, &realMessage, &evtDp);
		tmpWrapper = new ODOSLToken();
		THROW_IF_NULL(tmpWrapper);
		tmpWrapper->InitODOSLToken(ev);
		THROW_IF_ERROR( AEDescToODDesc(&evtDp, tmpWrapper ) );
		if ( !resolver->IsODToken( ev, tmpWrapper ) )
			THROW( errAENoSuchObject );
		ODDesc* myTokenODDesc;
		myTokenODDesc = resolver->GetUserToken(ev, tmpWrapper);
		error = ODDescToAEDesc(myTokenODDesc, &theToken );
	
		if (error == noErr)
		{
			switch (theToken.descriptorType)
			{
				case typeProperty:
					EditorPropAccessor* embedPropAccessorObj =
							FIRSTBYTESFROMHANDLE(theToken.dataHandle, EditorPropAccessor*);
					embedPropAccessorObj->SetData(&theData);
					delete embedPropAccessorObj;
					break;
			}
		}
		THROW_IF_ERROR(error);

	CATCH_ALL
		error = ErrorCode();
	ENDTRY
	
	AEDisposeDesc( &theToken );
	AEDisposeDesc( &theData );
	AEDisposeDesc( &evtDp );
	AEDisposeDesc( &realMessage );
	AEDisposeDesc( &realReply );
	ODDeleteObject( tmpWrapper );
	
	return error;
}	// HandleSetData()


//------------------------------------------------------------------------------
// HandleGetData
//------------------------------------------------------------------------------

ODError HandleGetData(				ODPart*			part,
									ODAppleEvent*	message,
									ODAppleEvent*	reply,
									ODPart*			editor)
{
	DescType			reqType;
	Size				theSize;
	ODError				error;
	Environment*		ev = somGetGlobalEnvironment();
	ODSession*			session = ODGetSession(ev, editor);
	ODNameResolver* 	resolver = session->GetNameResolver(ev);

	AEDesc				objectData = NULLDESC;
	AEDesc				evtDp = NULLDESC;
	AEDesc				token = NULLDESC;
	AppleEvent			realMessage = NULLDESC;
	AppleEvent			realReply = NULLDESC;
	THROW_IF_ERROR( ODDescToAEDesc(message, &realMessage));
	THROW_IF_ERROR( ODDescToAEDesc(reply, &realReply));

	ODOSLToken* tmpWrapper = kODNULL;
	
	TRY
		GetDirectParam( session, &realMessage, &evtDp);
		
		tmpWrapper = new ODOSLToken;
		THROW_IF_NULL(tmpWrapper);
		tmpWrapper->InitODOSLToken(ev);
		
		THROW_IF_ERROR( AEDescToODDesc(&evtDp, tmpWrapper ) );
		if ( !resolver->IsODToken( ev, tmpWrapper ) )
			THROW( errAENoSuchObject );
		
		ODDesc* myTokenODDesc;
		myTokenODDesc = resolver->GetUserToken(ev, tmpWrapper);
		THROW_IF_ERROR( ODDescToAEDesc(myTokenODDesc, &token ) );
	
				//	Next, get the requested return type, if it exists.
		error = AEGetParamPtr(&realMessage, keyAERequestedType, typeType,
						&reqType, (Ptr)&reqType, sizeof(reqType), &theSize);
		if (error == errAEDescNotFound)		// Not an error if return type is not found.
		{
			error = noErr;
			reqType = typeWildCard;
		}
		THROW_IF_ERROR(error);
	
		switch (token.descriptorType)
		{
			case typeProperty:
				EditorPropAccessor* propAccessorObj = FIRSTBYTESFROMHANDLE(
						token.dataHandle, EditorPropAccessor*);
				propAccessorObj->GetData(&objectData);
				delete propAccessorObj;
				break;
		}
		
		THROW_IF_ERROR( AEPutParamDesc( &realReply, keyAEResult, &objectData ) );
		THROW_IF_ERROR( AEDescToODDesc( &realReply, reply ) );
	CATCH_ALL
		error = ErrorCode();
	ENDTRY
	
	AEDisposeDesc( &token );
	AEDisposeDesc( &evtDp );
	AEDisposeDesc( &objectData );
	AEDisposeDesc( &realMessage );
	AEDisposeDesc( &realReply );
	ODDeleteObject( tmpWrapper );
	
	return error;
}	// HandleGetData()

