/*
	File:		TextEditorDef.h

	Contains:	Common definitions used by TextEditor.cpp & .r files

	Written by:	Steve Smith

	Copyright:	 1994-95 by Apple Computer, Inc., all rights reserved.
	
	-------------------------------------------------------------------
	
	Note:		This file will need to be modified for any derivitive
				work based on TextEditor. The areas marked with (CH)
				should be considered when making those changes.
*/

#ifndef _TEXTEDITORDEF_
#define _TEXTEDITORDEF_

// -- OpenDoc Includes --

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

//=====================================================================

// Class / Editor ID (CH)
#define kPartClassName					"TextEditor"
#define kTextEditorID					"SampleCode::"kPartClassName

// Kinds (CH)
#define kTextEditorKind					kODISOPrefix "Apple:Kind:TextEditor"
#define kSTXTDataKind					'stxt'
#define kTextDataKind					'TEXT'
#define kTextFileKind					'TEXT'

// User Strings (CH)
#ifdef qRezzing
	#define kTextEditorEditorUserString "TextEditor 1.0"
	#define kTextEditorKindUserString	"TextEditor Styled Text"
	#define kSTXTDataKindUserString		"Styled Text data"
	#define kTextDataKindUserString		"Text data"
	#define kTextFileKindUserString		"Text file"
#endif

// TextEditor OSTypes (CH)
#define kTextEditorEditorOSType			'TXED'
#define kTextEditorViewerOSType			'TXVW'	//  no good: re-register
#define kTextEditorDocumentOSType		'TXDC'
#define kTextEditorStationeryOSType		'sXDC'

// ISO Strings (CH)
#define kTextEditorPartInfo				kODISOPrefix "TextEditor:Display Frame Info"
#define kPropScriptRunnerShowingAnnotation	kODISOPrefix "TextEditor:ScriptRunner Showing"

#define	kMainPresentation  				kODISOPrefix "TextEditor:Presentation:Main"
#define kRulerPresentation				kODISOPrefix "TextEditor:Presentation:Ruler"

#define	kTEProp_Margins  				kODISOPrefix "TextEditor:Document:Margins"
#define	kTextEditorPreferences			kODISOPrefix "TextEditor:Preferences"

// TextEditor defines
#define	kBaseResourceID					20001

// 'nmap' Resource IDs
#define kKindCategoryMapId				kBaseResourceID+1
#define kEditorKindMapId				kBaseResourceID+2
#define kEditorUserStringMapId			kBaseResourceID+3
#define kKindUserStringMapId			kBaseResourceID+4
#define kOldMacOSTypeMapId				kBaseResourceID+6
#define kPlatformEditorKindMapId		kBaseResourceID+7

// Text Items
#define kRuntimeStringsID				kBaseResourceID
#define	kPrefFileNameStrIndex				1
#define	kAboutTextStrIndex        			2
#define	kPreferencesStrIndex				3
#define	kOriginalPreferencesStrIndex		4
#define	kShowRulerStrIndex					5
#define	kHideRulerStrIndex					6
#define	kShowScriptRunnerStrIndex			7
#define	kHideScriptRunnerStrIndex			8

// Error Messages
#define kErrorStringsID					kRuntimeStringsID+1
#define kErrCantInitializePart			1
#define kErrCantOpenDocWindow			2
#define kErrCantOpenPartWindow			3
#define kErrRemoveFrame					4
#define kErrWindowGone					5
#define kErrTranslationFailed			6
#define kErrTxCreateObjectFailed		7
#define kErrUnimplemented				8
#define kErrExternalizeFailed			9

// Bundles/FREFs
#define kDocumentBundle					kBaseResourceID
#define kEditorBundle					kBaseResourceID+1
#define kViewerBundle					kBaseResourceID+2
#define kDocumentFREF					kBaseResourceID
#define kStationeryFREF					kBaseResourceID+1
#define kEditorFREF						kBaseResourceID+2
#define kViewerFREF						kBaseResourceID+3

// Icons
#define	kLargeIcons						1
#define	kSmallIcons						2
#define	kDocumentIcons					kBaseResourceID
#define	kStationeryIcons				kBaseResourceID+1
#define	kEditorIcons					kBaseResourceID+2
#define	kViewerIcons					kBaseResourceID+3

// Pictures
#define kEditorIconPicture				kBaseResourceID
#define kDocIconPicture					kBaseResourceID+1
#define	kThumbnailPicture				kBaseResourceID+2
#define kLeftJustPicture				kBaseResourceID+599
#define kCenterJustPicture				kBaseResourceID+600
#define kRightJustPicture				kBaseResourceID+601
#define kFullJustPicture				kBaseResourceID+602
#define kRulerPicture					kBaseResourceID+603

// Dialogs & Windows
#define	kAboutDialogID					kBaseResourceID
#define kErrorBoxID						kBaseResourceID+1
#define	kMacWindowTitleBarHeight		20
#define	kALittleNudge					4
#define kMinVertVisPortion				10
#define kMinHorzVisPortion				16

#define	kDialogFontInfoID				kBaseResourceID
#define	kPreferencesDialogID			kBaseResourceID+1000
#define	kSettingsDialogID				kBaseResourceID+1001
#define	kSettingsNoMarginsDialogID		kBaseResourceID+1002
#define	kOtherSizeDialogID				kBaseResourceID+1003
#define kPopupControlID					kBaseResourceID+10
#define kPrefFontIndex					1
#define kSettingsFontIndex				2
#define kOtherFontIndex					3

// Menus
#define kBaseMenuID						kBaseResourceID
#define kNumMenus						5
#define	kFontMenuID						kBaseMenuID
#define	kSizeMenuID						kBaseMenuID+1
#define	kStyleMenuID					kBaseMenuID+2
#define	kFormatMenuID					kBaseMenuID+3
#define kToolsMenuID					kBaseMenuID+4
#define	kFontPopupMenuID				kBaseMenuID+10


// Display Frames
#define kFrameRemoved					1
#define kFrameClosed					0

// Geometry
#define kMinImagingResolution			72 // dpi

// Picture resources (CH)
//#define kRulerPictResID				20600
//#define kRulerBitmapResID				20600
#define kRulerPictResID					20604

// Commands (CH)
#define	kBaseFontCmdID					30000
#define	kBaseFontSizeCmdID				23000
#define	kBaseFontStyleCmdID				24000
#define	kShowHideRulerCmdID				25001
#define	kSettingsCmdID					25003
#define	kToggleScriptRunnerCmdID		25010

#define kOtherFontSizeCmdID				(kBaseFontSizeCmdID + 128)

// Misc (CH)
#define	kAll 							0L
#define kDontClone						0L
#define	kUndefined						-1

#define	kDefaultEmbeddedMargin			5	// pixels
#define	kDefaultDocMargin				72	// pixels
#define	kSingleScrollUnit				10

// Drag & Drop (CH)
#define kDragHighlightInset				-3

// Dialog controls (CH)
#define kErrStrFieldID					3

#endif

