/*------------------------------------------------------------------------------

	File:			TextEditorStorage.cpp
	
	Description:	TextEditor's storage protocol methods.
	
	Written by:		Steve Smith
	
	Copyright:		 1994 - 1995 by Apple Computer, Inc., all rights reserved.
	
------------------------------------------------------------------------------*/

// Notification that this is a SOM source file
#define SampleCode_TextEditor_Class_Prototypes
// define underscore (_) field names
#define VARIABLE_MACROS

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (e.g. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with "ev" parameter set.
#include <Except.h>
#endif

// -- TextEditor Includes

#ifndef SOM_SampleCode_TextEditor_xih
#include "TextEditor.xih"
#endif

#ifndef SOM_TextEditorSemIntf_xh
#include <TextEditorSemIntf.xh>
#endif

#ifndef _TEXTEDITORDEF_
#include "TextEditorDef.h"
#endif

#ifndef _TEXTEDITORUTILS_
#include "TextEditorUtils.h"
#endif

#ifndef _TEXTEDITORGLOBALS_
#include "TextEditorGlobals.h"
#endif

#ifndef _VALUESTREAM_
#include "ValueStream.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef SOM_ODMenuBar_xh
#include <MenuBar.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

#ifndef SOM_ODTypeList_xh
#include <TypeList.xh>
#endif

#ifndef SOM_ODTypeListIterator_xh
#include <TypLsItr.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _FOCUSLIB_
#include <FocusLib.h>
#endif

#ifndef _ODMEMORY_
#include <ODMemory.h>
#endif

#ifndef _USERSRCM_
#include <UseRsrcM.h>
#endif

#ifndef _ISOSTR_
#include <ISOStr.h>
#endif

#ifndef _ODDEBUG_
#include <ODDebug.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _ITEXT_
#include <IText.h>
#endif

#ifndef _STDTYPIO_
#include "StdTypIO.h"
#endif

#ifndef _STORUTIL_
#include "StorUtil.h"
#endif

#ifndef _TEMPITER_
#include <TempIter.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef _BNDNSUTL_
#include <BndNSUtl.h>
#endif

// -- Macintosh Includes --

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

// -- Textension Includes --

#ifndef _FileStream_
#include "FileStream.h"
#endif

#ifndef _Textension_
#include "Textension.h"
#endif

#ifndef _TSMTextension_
#include "TSMTextension.h"
#endif

#ifndef _TextensionTEScrap_
#include "TextensionTEScrap.h"
#endif

// -- ScriptRunner Includes --

#ifndef SOM_PaletteExt_xh
#include "PaletteExt.xh"
#endif

#ifndef _SCRIPTRUNNERSUPPORT_
#include "ScriptRunnerSupport.h"
#endif


//==============================================================================
// Class:		TextEditorIOSuite
//
// Description:	This is a Textension subclass that is used to handle I/O of
//				TextEdit-style Styled Text data.
//==============================================================================

class TextEditorIOSuite	: public CTextensionIOSuite {

public:
	TextEditorIOSuite() { };
	
	OSErr ITextEditorIOSuite(char local = true, CStream* stream = nil)
		{ return ITextensionIOSuite(local, stream); }
	
	// -- For Internalization --
	OSErr OutputText(void* textBuffer, long textSize);
	
	OSErr PutRunsFromTEStyles(StScrpPtr stylScrapPtr, long textSize);
	
	// -- For Externalization --
	OSErr GetText(void* outStylScrapPtr);

	OSErr GetStyles(StScrpPtr* outStylScrapPtr, long* styleSize);
};

//------------------------------------------------------------------------------
// Method:		OutputText
//
// Description:	This method should be called when text is being "output" _into_
//				an object of this class.  Once this is called, the object will
//				contain the text that was passed in as its text data.
//------------------------------------------------------------------------------

OSErr TextEditorIOSuite::OutputText( void* textBuffer, long textSize )
{
	OSErr err = this->PutText(textBuffer, textSize);
	this->SizeChanged();
	
	return err;
}

//------------------------------------------------------------------------------
// Method:		PutRunsFromTEStyles
//
// Description:	This method should be called when style information is being 
//				"output" _into_ an object of this class.  This should be called
//				after the text has been added by OutputText.  Once this is 
//				called,  the object will contain the text that it had before
//				this operation, but the text will now be styled with the
//				appropriate styles.
//------------------------------------------------------------------------------

OSErr TextEditorIOSuite::PutRunsFromTEStyles( StScrpPtr	stylScrapPtr, 
											  long		textSize )
{
	register short countStyles = stylScrapPtr->scrpNStyles;
	if (!countStyles) return noErr;
	
	OSErr err = this->NewIOSuiteType(kRunsIOSuiteType, kLocalObjectIOSuiteSize * countStyles);
	if (err) return err;

	register ScrpSTElement* currStyleElem = (ScrpSTElement*) &stylScrapPtr->scrpStyleTab;
	register ScrpSTElement* nextStyleElem = currStyleElem;
	
	do 
	{
		long runLen;
		
		++nextStyleElem;
		
		runLen = ((countStyles == 1) ? textSize : nextStyleElem->scrpStartChar)
							- currStyleElem->scrpStartChar;
		
		CRunObject* runObject = CTextensionTEScrap::TEScrapStyleToTextObject(*currStyleElem);
		if (!runObject) {
			err = memFullErr;
			break;
		}
		
		err = this->PutObjectRange(runObject, runLen);
		if (err) break;
		
		currStyleElem = nextStyleElem;
	} 
	while (--countStyles);
	
	this->SizeChanged();
	
	return err;
}

//------------------------------------------------------------------------------
// Method:		GetStyles
//
// Description:	The the TextEdit-style Style information from the text that is
//				contained in this IOSuite.  The pointer that is returned in
//				the first paramater has been allocated with ODNewPtr, and must
//				be disposed by the caller.
//------------------------------------------------------------------------------

OSErr TextEditorIOSuite::GetStyles( StScrpPtr* outStylScrapPtr, long* styleSize )
{
	*outStylScrapPtr = kODNULL;
	*styleSize = 0;

	TDataHeader header;
	OSErr err = this->SearchIOSuiteType(kRunsIOSuiteType, &header);
	if ( err ) 
	{
		if ( err == noTypeErr ) 
			err = noErr;
		return err;
	}
	
	long countStyles = header.dataSize / kLocalObjectIOSuiteSize;
	if ( countStyles > kMaxInt )
		return noErr; //'styl' scrap has a short count for styles
	
	long stylScrapSize = sizeof(short) + countStyles * sizeof(ScrpSTElement);
	
	*outStylScrapPtr = (StScrpPtr) ODNewPtr(stylScrapSize);

	StScrpPtr stylScrapPtr = *outStylScrapPtr;
	stylScrapPtr->scrpNStyles = (short) countStyles;
	
	register ScrpSTElement* currStyleElem = (ScrpSTElement*) &stylScrapPtr->scrpStyleTab;
	register long currOffset = 0;
	char firstTextRun = 1;
	
	do 
	{
		long rangeLen;
		CRunObject* rangeObj;
		
		err = this->GetObjectRange(header.dataType, (CAttrObject**) &rangeObj, 
								   &rangeLen);
		if ( err ) 
			break;
		
		if ( rangeObj->IsTextRun() ) 
		{ 
			CTextensionTEScrap::TextObjectToTEScrapStyle(rangeObj, currStyleElem);
			
			if (firstTextRun) 
			{
				firstTextRun = 0;
				currStyleElem->scrpStartChar = 0;
			}
			else 
			{
				currStyleElem->scrpStartChar = currOffset;
			}
			
			++currStyleElem;
		}
		else 
		{
			--stylScrapPtr->scrpNStyles;
		}
		
		currOffset += rangeLen;
	} 
	while (--countStyles);
	
	if ( !err )
	{
		// Shring the size of the block if not all runs had style information.
		long finalStylScrapSize = sizeof(short) + stylScrapPtr->scrpNStyles 
										* sizeof(ScrpSTElement);
		if ( finalStylScrapSize != stylScrapSize )
			*outStylScrapPtr = (StScrpPtr) ODReallocate(*outStylScrapPtr, 
														finalStylScrapSize);
		*styleSize = finalStylScrapSize;
	}
	else
	{
		ODDisposePtr(*outStylScrapPtr);
		*outStylScrapPtr = kODNULL;
		*styleSize = 0;
	}

	return err;
}

//==============================================================================
// Multiple Kind Support utility functions
//==============================================================================

//------------------------------------------------------------------------------
// Function:	IsKindSupported
// Origin:		TextEditor
//
// Description:	Check to see if this part editor supports the kind in
//				question.
//------------------------------------------------------------------------------

static ODBoolean IsKindSupported( Environment*		ev, 
								  ODTypeToken		token )
{
	return (token == gGlobals->fNativeData
		 || token == gGlobals->fImportedData
		 || token == gGlobals->fStyledTextData);
}


//------------------------------------------------------------------------------
// Method:		GetKindFromToken
// Origin:		TextEditor
//
// Description:	Given a token, return the type that it corresponds to.
//------------------------------------------------------------------------------

static ODType GetKindFromToken( Environment*		ev, 
								ODTypeToken			token)
{
	WASSERT(IsKindSupported(ev, token));

	if ( token == gGlobals->fNativeData )
		return kTextEditorKind;

	else if ( token == gGlobals->fImportedData )
		return gGlobals->fScrapTextValue;
	
	else if ( token == gGlobals->fStyledTextData )
		return gGlobals->fStyledTextKind;
	
	else
		return kODNULL;
}

//------------------------------------------------------------------------------
// Method:		SetPreferredKind
// Origin:		TextEditor
//
// Description:	This function will set the preferred kind property of the
//				given storage unit to the given type.
//------------------------------------------------------------------------------

static void SetPreferredKind( Environment*		ev, 
							  ODStorageUnit*	storageUnit,
							  ODType			kind )
{
	if ( kind == kODNULL )
	{
		ODSURemoveProperty(ev, storageUnit, kODPropPreferredKind);
	}
	else
	{
		TRY
			ODSetISOStrProp(ev, storageUnit, kODPropPreferredKind, kODISOStr, kind);
		CATCH_ALL
			// Remove the property and value if something went wrong.
			ODSURemoveProperty(ev, storageUnit, kODPropPreferredKind);
		ENDTRY
	}
}

//------------------------------------------------------------------------------
// Function:	GetNextHighestFidelity
//
// Description:	Returns the type token that is of the next highest fidelity
//				of the kinds we support.
//
//				If kODNullTypeToken is passed in, this returns the highest
//				fidelity.
//
//				If the lowest fidelity is passed in, this returns 
//				kODNullTypeToken.
//------------------------------------------------------------------------------

static ODTypeToken GetNextHighestFidelity( ODTypeToken currentKind = kODNullTypeToken )
{
	ODTypeToken nextKind;

	if ( currentKind == kODNullTypeToken )
		nextKind = gGlobals->fNativeData;
	
	if ( currentKind == gGlobals->fNativeData )
		nextKind = gGlobals->fStyledTextData;

	else if ( currentKind == gGlobals->fStyledTextData )
		nextKind = gGlobals->fImportedData;

	else if ( currentKind == gGlobals->fImportedData )
		nextKind = kODNullTypeToken;
		
	else
		// It's something we don't support, so start at the top.
		nextKind = gGlobals->fNativeData;
	
	return nextKind;
}

//------------------------------------------------------------------------------
// Function:	GetKindToInternalize
// Origin:		SoundEditor
//
// Description:	Returns a token representing the kind of data that should be
//				internalized from the given storage unit.
//------------------------------------------------------------------------------

static ODTypeToken GetKindToInternalize( Environment*		ev,
										 ODSession*			session, 
										 ODStorageUnit*		storageUnit )
{
	// This should become a token representing the kind of data that should be
	// internalized from the given storage unit.
	ODTypeToken kindToUse = kODNullTypeToken;

	TempODISOStr docPrefKindStr = ODGetISOStrProp(ev, storageUnit, kODPropPreferredKind, 
											 	  kODISOStr, kODNULL, kODNULL);		
	
	ODTypeToken docPrefKind = kODNullTypeToken;
	if (docPrefKindStr != kODNULL)
		docPrefKind = session->Tokenize(ev, docPrefKindStr);
	
	// If the data has a preferred kind, see if we can use it.
	if ( docPrefKind != kODNullTypeToken && IsKindSupported(ev, docPrefKind)
		 && storageUnit->Exists(ev, kODPropContents, GetKindFromToken(ev, docPrefKind), 0) )
	{
		kindToUse = docPrefKind;
	}
	
	// If there is not document-preferred kind, we don't support it, or it wasn't found,
	// find the first value (the one with highest fidelity) that we understand.
	if ( kindToUse == kODNullTypeToken )
	{
		for ( ODTypeToken kindToLookFor = GetNextHighestFidelity();
			  kindToLookFor != kODNullTypeToken;
			  kindToLookFor = GetNextHighestFidelity(kindToLookFor) )
		{
			// Note: we have to focus here (not just Exists()), or for some reason, 
			// importing doesn't work correctly.  Instead, a translation dialog 
			// comes up and the drag (if that's what it is) is refused.
			if ( ODSUExistsThenFocus(ev, storageUnit, kODPropContents, 
									 GetKindFromToken(ev, kindToLookFor)) )
			{
				kindToUse = kindToLookFor;
				break;
			}
		}
	}

	return kindToUse;
}

//------------------------------------------------------------------------------
// Function:	HasHigherFidelity
//
// Description:	Returns whether the first parameter (typeInQuestion) is of a 
//				higher fidelity than the second paramater (compareToType).  
//
//				It will return kODFalse if the first parameter is not a
//				supported kind (it is not of a higher fidelity), and kODTrue
//				if the first parameter is supported but the second is not.
//------------------------------------------------------------------------------

static ODBoolean
HasHigherFidelity( Environment*		ev, 
				   ODTypeToken		typeInQuestion, 
				   ODTypeToken		compareToType )
{
	ODBoolean isHigher = kODFalse;
	
	ODTypeToken type = typeInQuestion;
	
	// Iterate over all of the types of fidelity lower than typeInQuestion.
	do 
	{
		type = GetNextHighestFidelity(type);
		
		// If any one of them is the compareToType, then the typeInQuestion
		// is of higher fidelity than compareToType.
		if ( type == compareToType )
			isHigher = kODTrue;
	}
	while ( type != kODNullTypeToken && isHigher == kODFalse );
	
	return isHigher;
}

//------------------------------------------------------------------------------
// Function:	ForceFulfillPromise
//
// Description:	If the focused value of the storage unit passed in is a 
//				promise, force it to be fulfilled.  (We do this simply by
//				calling GetSize, but use this function to indicate what
//				operation is actually being performed for clarity in calling
//				code.)
//------------------------------------------------------------------------------

static void ForceFulfillPromise( Environment*		ev, 
								 ODStorageUnit*		su )
{
	(void) su->GetSize(ev);
}

//------------------------------------------------------------------------------
// Function:	InsertStyledTextData
//
// Description:	This function should be called when there is 'stxt' formatted
//				data available in a pointer in memory and it is to be inserted 
//				into a specified place in the text.
//------------------------------------------------------------------------------

static ODULong InsertStyledTextData( Environment*		ev, 
									 CTextension*		textension,
									 ODPtr				styledTextPtr,
									 ODSize				styledTextSize,
									 TOffsetRange*		selRange )
{
	ODULong totalBytesRead = 0;

	TextEditorIOSuite* ioSuite = new TextEditorIOSuite;
	ioSuite->ITextEditorIOSuite();
	ioSuite->Empty();
	
	TRY
		// Get the sizes of the style and text parts of the 'stxt' format.
		ODSShort numStyles;
		numStyles = *(ODSShort*) styledTextPtr;
		
		ODSLong stylSize = (numStyles * sizeof(ScrpSTElement)) + sizeof(short);
		ODSLong textSize = styledTextSize - stylSize;
		
		// Get the text data and put it into the IOSuite.
		{
			totalBytesRead += textSize;
			if ( textSize > 0 )
			{
				void* textPtr = (char*) styledTextPtr + stylSize;
				THROW_IF_ERROR(ioSuite->OutputText(textPtr, textSize));
			}
		}
		
		// Get the styles and apply them to the text in the IOSuite.
		{
			totalBytesRead += stylSize;
			if (stylSize > 0)
				THROW_IF_ERROR(ioSuite->PutRunsFromTEStyles((StScrpPtr) styledTextPtr, textSize));
		}
		
		// Get the range to replace, if any.
		ODULong start = selRange ? selRange->Start() : 0;
		ODULong end   = selRange ? selRange->End()   : 0;
		
		// Replace with imported text.
		const TReplaceParams replaceParams(ioSuite);
		THROW_IF_ERROR(textension->ReplaceRange(start, end, replaceParams));
		
	CATCH_ALL
		ioSuite->Free();
		delete ioSuite;
		RERAISE;
	ENDTRY

	ioSuite->Free();
	delete ioSuite;
	
	return totalBytesRead;
}

//------------------------------------------------------------------------------
// Function:	InsertStyledTextData
//
// Description:	This function should be called when there is 'stxt' formatted
//				data available in a storage unit and it is to be inserted into
//				a specified place in the text.
//------------------------------------------------------------------------------

static ODULong InsertStyledTextData( Environment*		ev, 
									 CTextension*		textension,
									 ODStorageUnitView*	suView,
									 TOffsetRange*		selRange )
{
	ODULong totalBytesRead = 0;

	TextEditorIOSuite* ioSuite = new TextEditorIOSuite;
	ioSuite->ITextEditorIOSuite();
	ioSuite->Empty();
	
	TRY
		// Get the sizes of the style and text parts of the 'stxt' format.
		ODSShort numStyles;
		StorageUnitViewGetValue(suView, ev, sizeof(short), &numStyles);
		ODULong stylSize = (numStyles * sizeof(ScrpSTElement)) + sizeof(short);
		ODULong textSize = suView->GetSize(ev) - stylSize;
		
		// Get the text data and put it into the IOSuite.
		{
			suView->SetOffset(ev, stylSize);
			
			TempODByteArrayStruct ba;
			ODULong bytesRead = suView->GetValue(ev, textSize, ba);
			totalBytesRead += bytesRead;
			if ( bytesRead > 0 )
				THROW_IF_ERROR(ioSuite->OutputText(ba->_buffer, bytesRead));
		}
		
		// Get the styles and apply them to the text in the IOSuite.
		{
			suView->SetOffset(ev, 0);
			
			TempODByteArrayStruct ba;
			ODULong bytesRead = suView->GetValue(ev, stylSize, ba);
			totalBytesRead += bytesRead;
			if (bytesRead > 0)
				THROW_IF_ERROR(ioSuite->PutRunsFromTEStyles((StScrpPtr) ba->_buffer, textSize));
		}
		
		// Get the range to replace, if any.
		ODULong start = selRange ? selRange->Start() : 0;
		ODULong end   = selRange ? selRange->End()   : 0;
		
		// Replace with imported text.
		const TReplaceParams replaceParams(ioSuite);
		THROW_IF_ERROR(textension->ReplaceRange(start, end, replaceParams));
		
	CATCH_ALL
		ioSuite->Free();
		delete ioSuite;
		RERAISE;
	ENDTRY

	ioSuite->Free();
	delete ioSuite;
	
	return totalBytesRead;
}

//------------------------------------------------------------------------------
// Method:		ExportStyledTextData
// Origin:		TextEditor
//------------------------------------------------------------------------------

static void ExportStyledTextData ( Environment*			ev,
								   CTextension*			textension,
								   ODStorageUnitView*	suView,
								   TOffsetRange*		selRange )
{
	SOM_TRY
	
		// We are writing out styled text data
		
		TextEditorIOSuite *ioSuite = new TextEditorIOSuite;
		ioSuite->ITextEditorIOSuite();
		
		// Write the selection into our IOSuite so it's in a format that
		// we can then translate.
		if ( selRange )
		{
			THROW_IF_ERROR(textension->OutputRangeIOSuite(selRange, 
														  ioSuite, kIOAll));
		}
		else
		{
			TOffsetRange range2Save(0, textension->CountCharsBytes());
			THROW_IF_ERROR(textension->OutputRangeIOSuite(&range2Save, 
														  ioSuite, kIOAll));
		}
		
		// Get the style information and externalize it.
		{
			StScrpPtr stylScrapPtr = kODNULL;
			TempODPtr disposer = stylScrapPtr;
			long size;
			
			THROW_IF_ERROR(ioSuite->GetStyles(&stylScrapPtr, &size));
			StorageUnitViewSetValue(suView, ev, size, stylScrapPtr);
		}
		
		// Get the text range and externalize it.
		{
			long start = selRange ? selRange->Start() : 0;
			long bytes = selRange ? (selRange->End() - selRange->Start())
								  : (textension->CountCharsBytes());
			
			StorageUnitViewSetValue(suView, ev, bytes,
									textension->GetCharPtr(start));
		}
			
		// Free the ioSuite.
		ioSuite->Free();
		delete ioSuite;
	
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		InsertNativeTextensionData
// Origin:		TextEditor
//------------------------------------------------------------------------------

static ODULong InsertNativeTextensionData( Environment*		ev,
										   CTextension*		textension,
										   CStream*			stream,
										   TOffsetRange*	selRange )
{
	ODSLong bytesRead = 0;

	SOM_TRY

		bytesRead = stream->GetSize();
		if ( bytesRead > 0 )
		{
			// Set the range to replace, if needed.
			
			// Get the range to replace, if any.
			ODULong start = selRange ? selRange->Start() : 0;
			ODULong end   = selRange ? selRange->End()   : 0;

			CTextensionIOSuite *ioSuite = kODNULL;
			
			TRY
				ioSuite = new CTextensionIOSuite;
				ioSuite->ITextensionIOSuite(false, stream);
					
				const TReplaceParams replaceParams(ioSuite);
				THROW_IF_ERROR(textension->ReplaceRange(start, end, replaceParams));
			CATCH_ALL
				ioSuite->Free();
				delete ioSuite;
				RERAISE;
			ENDTRY
			
			ioSuite->Free();
			delete ioSuite;
		}
		
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return bytesRead;
}

//------------------------------------------------------------------------------
// Method:		InsertPlainTextData
// Origin:		TextEditor
//------------------------------------------------------------------------------

static ODULong InsertPlainTextData( Environment*		ev,
									CTextension*		textension,
									CStream*			stream,
									TOffsetRange*		selRange )
{
	ODSLong bytesRead = 0;

	SOM_TRY

		bytesRead = stream->GetSize();
		if ( bytesRead > 0 )
		{
			// Get the range to replace, if any.
			ODULong start = selRange ? selRange->Start() : 0;
			ODULong end   = selRange ? selRange->End()   : 0;

			// Read in generic Text data.
			TTextDescriptor textDesc(stream);
			const TReplaceParams replaceParams(textDesc);
			
			THROW_IF_ERROR(textension->ReplaceRange(start, end, replaceParams));
		}
		
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return bytesRead;
}

//------------------------------------------------------------------------------
// Method:		InsertTextFile
// Origin:		TextEditor
//------------------------------------------------------------------------------

static ODULong InsertTextFile( Environment*			ev,
							   CTextension*			textension,
							   FSSpec&				file,
							   TOffsetRange*		selRange )
{
	ODSLong totalBytesRead = 0;
	
	SOM_TRY

		ODSShort fileRefNum;
		THROW_IF_ERROR(FSpOpenDF(&file, fsRdPerm, &fileRefNum));
		
		TRY
			ODSLong textSize;
			THROW_IF_ERROR(GetEOF(fileRefNum, &textSize));
			
			// If the file is empty, there's no reason to try to read it.
			if ( textSize > 0 )
			{
				TextEditorIOSuite* ioSuite = new TextEditorIOSuite;
				ioSuite->ITextEditorIOSuite();
				ioSuite->Empty();
				
				TRY
					// Get the text data and put it into the IOSuite.
					{
						TempODPtr text = ODNewPtr(textSize);
						
						ODSLong bytesRead = textSize;
						FSRead(fileRefNum, &bytesRead, text);
						totalBytesRead += bytesRead;
						
						if ( bytesRead > 0 )
							THROW_IF_ERROR(ioSuite->OutputText(text, bytesRead));
					}
					
					// Get the styles from a resource if there are any and 
					// apply them to the text in the IOSuite.
					{
						short oldResRef = CurResFile();
						short resFileRef = FSpOpenResFile(&file, fsRdPerm);
						TRY
							if ( resFileRef != -1 && Count1Resources('styl') > 0 )
							{
								Handle stylHandle = Get1IndResource('styl', 1);
																
								if (stylHandle && GetHandleSize(stylHandle) > 0)
								{
									totalBytesRead += GetHandleSize(stylHandle);
		
									HLockHi(stylHandle);
									OSErr err = ioSuite->PutRunsFromTEStyles((StScrpPtr) *stylHandle, textSize);
									HUnlock(stylHandle);
									ReleaseResource(stylHandle);
		
									THROW_IF_ERROR(err);
								}
							}
						CATCH_ALL
							if ( resFileRef != -1 )
								CloseResFile(resFileRef);
							UseResFile(oldResRef);
							RERAISE;
						ENDTRY
						
						if ( resFileRef != -1 )
							CloseResFile(resFileRef);
						UseResFile(oldResRef);
					}
					
					// Get the range to replace, if any.
					ODULong start = selRange ? selRange->Start() : 0;
					ODULong end   = selRange ? selRange->End()   : 0;
					
					// Replace with imported text.
					const TReplaceParams replaceParams(ioSuite);
					THROW_IF_ERROR(textension->ReplaceRange(start, end, replaceParams));
					
				CATCH_ALL
					ioSuite->Free();
					delete ioSuite;
					RERAISE;
				ENDTRY
			
				ioSuite->Free();
				delete ioSuite;
			}

		CATCH_ALL
			FSClose(fileRefNum);
			RERAISE;
		ENDTRY
		
		FSClose(fileRefNum);

	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return totalBytesRead;
}

//==============================================================================
// TextEditor methods
//==============================================================================

//------------------------------------------------------------------------------
// Method:		InternalizeStateInfo
// Origin:		TextEditor
//
// Description:	This method is used to read in "state" information for the part.
//				This is information related to the workings of the part editor,
//				not the content.
//
//				The part rights out a list of weak references to its display
//				frames. This allows the part to reuse the same display frames
//				each time the document is opened. Those references are read
//				back in and validated here.
//
// Note:		The function StorageUnitGetValue simplifies the use of
//				ODByteArrary, which is required the StorageUnit interface. Look
//				in StorUtil.h/cpp for an example of using the ODByteArray struct.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__InternalizeStateInfo
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","InternalizeStateInfo");

	SOM_TRY
	
		// Internalize information about ScriptRunner's state.
		
		_fIsScriptRunnerOn = kODFalse;
		
		if ( ODSUExistsThenFocus(ev, storageUnit, kPropScriptRunnerShowingAnnotation, kODBoolean) )
			_fIsScriptRunnerOn = ODGetBooleanProp(ev, storageUnit, 
													   kPropScriptRunnerShowingAnnotation, 
													   kODBoolean);
		
		// If it should be on, we need to create it (if it's still present) and
		// show the floater if we're activated.
		if ( _fIsScriptRunnerOn && !_fReadOnlyStorage )
		{
			if ( _fScriptPaletteExt == kODNULL &&
				 IsScriptRunnerInstalled(ev, ODGetSession(ev, _fSelf)) )
			{
				_fScriptPaletteExt = AcquireScriptRunner(ev, ODGetDraft(ev, somSelf));
				_fScriptPaletteExt->SetClient(ev, _fSelf);
			}
		
			if ( _fScriptPaletteExt )
			{
				// We should only do this if we are the active part.
				TempODFrame activeFrame = ODGetSession(ev, somSelf)->GetArbitrator(ev)
								->AcquireFocusOwner(ev, gGlobals->fSelectionFocus);
				TempODPart activePart = activeFrame ? activeFrame->AcquirePart(ev) : kODNULL;
				
				// Determine if we need to update the state of the flag.
				if ( ODObjectsAreEqual(ev, activePart, _fSelf) )
				{
					_fScriptPaletteExt->Show(ev);
					_fIsScriptRunnerHidden = kODFalse;
				}
			}
		}
		
		// Internalize the part's display frame list.
	
		if ( ODSUExistsThenFocus(ev, storageUnit, kODPropDisplayFrames, kODWeakStorageUnitRefs) )
		{
			ODULong size = storageUnit->GetSize(ev);
			storageUnit->SetOffset(ev, 0);	
		
			for ( ODULong offset = 0; offset < size; offset += kODStorageUnitRefSize )
			{	
				TRY
					ODStorageUnitRef weakRef;
					StorageUnitGetValue(storageUnit, ev, kODStorageUnitRefSize, (ODPtr)&weakRef);
					
					if ( storageUnit->IsValidStorageUnitRef(ev, weakRef) )
					{		
						// We lazily internalize our display frames, meaning we don't "get"
						// the frame until we absolutely need it. This reduces the time
						// to internalize the part and the amount of memory needed.
						
						// Convert the reference into a runtime id.
						ODID frameID = storageUnit->GetIDFromStorageUnitRef(ev, weakRef);
	
						// Create a proxy class to support the lazy internalization.
						CFrameProxy* proxy = new CFrameProxy;
						proxy->InitFrameProxy(frameID, ODGetDraft(ev,storageUnit));
	
						// Add the proxy to the display frame collection.
						_fDisplayFrames->Add(proxy);
					}
			//		else
    		//			WARN("Invalid StorageUnitRef read.");
				CATCH_ALL
					// consume exception
				ENDTRY
			}
		}
		
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		InternalizeContent
// Origin:		TextEditor
//
// Description:	This method is called during initialization of the part from an
//				existing document. The content of the part should be read in.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	ODULong
SOMLINK		TextEditor__InternalizeContent
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit,
				ODPtr						selRange
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","InternalizeContent");

	ODSLong bytesRead = 0;

	SOM_TRY
		ODTypeToken kindToUse = GetKindToInternalize(ev, ODGetSession(ev, somSelf), 
													 storageUnit);

		// Read the kind from the storage unit.

		if ( kindToUse == gGlobals->fNativeData
			 && ODSUExistsThenFocus(ev, storageUnit, kODPropContents, kTextEditorKind) )
		{
			// Here we call ForceFulfillPromise which will fulfill the promised data
			// into the focused value of the storage unit.  We need to do this because
			// of a problem in OpenDoc where a second ODStorageUnitView is created to 
			// fulfill the promise, but it will throw because our ODStorageUnitView
			// already has the storage unit locked.
			ForceFulfillPromise(ev, storageUnit);
				
			// Read in native Textension data format
			ODStorageUnitView* suView = storageUnit->CreateView(ev);
			bytesRead = somSelf->StreamDataIn(ev, suView, selRange, kIOAll);
		}
		else if ( kindToUse == gGlobals->fImportedData
				  && ODSUExistsThenFocus(ev, storageUnit, kODPropContents, gGlobals->fScrapTextValue) )
		{
			ForceFulfillPromise(ev, storageUnit);
				
			// Read in a plain text value for compatability with text editors
			ODStorageUnitView* suView = storageUnit->CreateView(ev);
			bytesRead = somSelf->StreamDataIn(ev, suView, selRange, kNoIOFlags);
		}
		else if ( kindToUse == gGlobals->fStyledTextData
				  && ODSUExistsThenFocus(ev, storageUnit, kODPropContents, gGlobals->fStyledTextKind) )
		{
			ForceFulfillPromise(ev, storageUnit);
			
			// Read in a styled text ('stxt') value for compatability with text editors
			TempODStorageUnitView suView = storageUnit->CreateView(ev);
			bytesRead = InsertStyledTextData(ev, _fTextension, suView, (TOffsetRange*) selRange);
		}
		else if ( kindToUse == kODNullTypeToken
				  && ODSUExistsThenFocus(ev, storageUnit, kODPropContents, gGlobals->fAppleHFSFlavor) )
		{
			// Read a plain TEXT file for compatability with other apps.
			// Note: This must be the last value that is checked for because it will
			// exist if an OpenDoc file with one of the other values was dropped in.
			
			ODSLong size = storageUnit->GetSize(ev);
	
			if ( size > 0 )
			{
				HFSFlavor* hfsData = (HFSFlavor*) ODNewPtr(size);
				TempODPtr disposer = hfsData;
				
				StorageUnitGetValue(storageUnit, ev, size, (ODPtr) hfsData);
				
				if (hfsData->fileType == kTextFileKind)
				{
					bytesRead = InsertTextFile(ev, _fTextension, hfsData->fileSpec,
											   (TOffsetRange*) selRange);
				}
			}
		}

		// If we don't have a preferred kind yet, use the kind that was found, 
		// if any.  Otherwise, use our default kind.
		if ( _fPreferredKind == kODNullTypeToken )
			_fPreferredKind = (kindToUse != kODNullTypeToken) ? kindToUse 
															  : gGlobals->fNativeData;
			
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return bytesRead;
}

//------------------------------------------------------------------------------
// Method:		CloneInto
// Origin:		ODPart
//
// Description:	This method is called during cloning, typically during data
//				interchange operations (e.g. Cut/Paste). The part should
//				write out its current state and content.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__CloneInto
 			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODDraftKey					key,
				ODStorageUnit*				destinationSU,
				ODFrame*					initiatingFrame
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","CloneInto");
	
	SOM_TRY
	
		// This calls back up to the persistent object parent class
		// which copies over our timedate stamp and last modified info.
		SampleCode_TextEditor_parent_ODPart_CloneInto(somSelf,ev, key, destinationSU, initiatingFrame);
	
		// If we haven't written out to this storage unit, externalize the
		// current state and contents.
		
		if ( !destinationSU->Exists(ev, kODPropContents, kTextEditorKind, 0) &&
			 !destinationSU->Exists(ev, kODPropContents, gGlobals->fScrapTextValue, 0) &&
			 !destinationSU->Exists(ev, kODPropContents, gGlobals->fStyledTextKind, 0) )
		{
			// Add the properties we need to successfully externalize
			// ourselves into the destination storage unit.
			somSelf->CheckAndAddProperties(ev, destinationSU);
					
			// Write out the part's state information.
			somSelf->ExternalizeStateInfo(ev, destinationSU, key, initiatingFrame);
				
			// Write out the part's content.
			somSelf->ExternalizeContent(ev, destinationSU, kODNULL);
		}
				
	SOM_CATCH_ALL
	SOM_ENDTRY
}
	
//------------------------------------------------------------------------------
// Method:		Externalize
// Origin:		ODPersistentObject
//
// Description:	This method is called when the user saves the document. The part
//				should write out its state and content if changes have occurred
//				and if our storage is writeable.
//
// Note:		The part must call its parent class behavior because one or more
//				of our parent classes contains implementation. This is done in
//				the SOM class.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__Externalize
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","Externalize");

	SOM_TRY
	
		// First, ask our parent classes to externalize themselves.
		SampleCode_TextEditor_parent_ODPart_Externalize(somSelf, ev);
		
		if (_fDirty)
		{
			// Get our storage unit.
			ODStorageUnit* storageUnit = somSelf->GetStorageUnit(ev);
	
			// Verify that the storage unit has the appropriate properties
			// and values to allow us to run. If not, add them.
			somSelf->CheckAndAddProperties(ev, storageUnit);
	
			// Verify that there are no "bogus" values in the Content
			// property.
			somSelf->CleanseContentProperty(ev, storageUnit);
			
			// Write out the part's state information.
			somSelf->ExternalizeStateInfo(ev, storageUnit, kODNULLKey, kODNULL);
			
			// Write out the part's content.
			somSelf->ExternalizeContent(ev, storageUnit, kODNULL);
	
			_fDirty = kODFalse;
		}
			
	SOM_CATCH_ALL
		// Alert the user of the problem.
		somSelf->DoDialogBox(ev, kODNULL, kErrorBoxID, kErrExternalizeFailed);
		// Change the exception value, so the DocShell doesn't display an
		// error dialog.
		SetErrorCode(kODErrAlreadyNotified);
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ExternalizeStateInfo
// Origin:		TextEditor
//
// Description:	This method is called during externalization of the part. The
//				current "state" of the part should be written out. This "state"
//				information may be lost during Data Interchange operations, so
//				the part needs to recover gracefully if information is missing
//				or incomplete.
//
// Note:		The function StorageUnitSetValue is a macro which simplifies
//				the use of ODByteArrary, which is required the StorageUnit
//				interface. Look in StorUtil.h/cpp for an example of using the
//				ODByteArray struct.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__ExternalizeStateInfo
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit,
				ODDraftKey					key,
				ODFrame*					scopeFrame
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","ExternalizeStateInfo");

	SOM_TRY
	
		// Set up the preferred kind property with our preferred kind.
		SetPreferredKind(ev, storageUnit, GetKindFromToken(ev, _fPreferredKind));
		
		// Write out information about ScriptRunner's state.
		somSelf->UpdateScriptRunnerState(ev);
		ODSetBooleanProp(ev, storageUnit, kPropScriptRunnerShowingAnnotation, 
						 kODBoolean, _fIsScriptRunnerOn);
		
		// Set up the preferred kind property with our preferred kind.
		SetPreferredKind(ev, storageUnit, GetKindFromToken(ev, _fPreferredKind));

		// Externalize the part's display frame list.
		ODID		scopeFrameID = ( scopeFrame ? scopeFrame->GetID(ev) : kODNULLID );
		ODDraft*	fromDraft = ODGetDraft(ev, somSelf);
		
		storageUnit->Focus(ev, kODPropDisplayFrames, kODPosUndefined,
						   kODWeakStorageUnitRefs, 0, kODPosUndefined);
			
		// Persistent object references are stored in a side table, rather than
		// in the property/value stream. Thus, deleting the contents of a value
		// will not "delete" the references previously written to that value. To
		// completely "delete" all references written to the value, we must
		// remove the value and add it back.
		storageUnit->Remove(ev);
		storageUnit->AddValue(ev, kODWeakStorageUnitRefs);
	
		CListIterator fiter(_fDisplayFrames);
		for ( CFrameProxy* proxy = (CFrameProxy*) fiter.First();
				fiter.IsNotComplete(); proxy = (CFrameProxy*) fiter.Next() )
		{
			// Get the ID of the frame we are going to weakly reference.
			ODID frameID = proxy->GetID();
			
			// If a draft key exists, then we are being cloned to another draft.
			// We must "weak" clone our display frame and reference the cloned
			// frame. The part re-uses the frameID variable so there aren't two
			// different GetWeakStorageUnitRef calls.
			if ( key )
				frameID = fromDraft->WeakClone(ev, key, frameID, kODNULLID, scopeFrameID);
			
			// Write out weak references to each of the part's display frames.
			ODStorageUnitRef weakRef;
			storageUnit->GetWeakStorageUnitRef(ev, frameID, weakRef);
			TRY 
				StorageUnitSetValue(storageUnit, ev, kODStorageUnitRefSize, (ODPtr)&weakRef);
			CATCH_ALL
				// consume the exception
			ENDTRY
		}
		
		// write out paper size
		// write out page size
		// write out page margins
		// write out text alignment
		// write out print record
				
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ExternalizeContent
// Origin:		TextEditor
//
// Description:	This method is called during exteralization of the part. The
//				content of the part should be written out.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__ExternalizeContent
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit,
				ODPtr						selRange
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","ExternalizeContent");

	SOM_TRY
	
		// If we have interchange pref, write all kinds.
	
		if (_fTextensionInitialized)
		{
			ODSLong start, end;
			
			// If there is a "open" inline hole, we need to close it
			// before saving.
			if ( _fTextension->HasActiveInputArea(&start, &end) )
				_fTextension->FixActiveInputArea();
			
			// Write out native Textension data format
			if ( ODSUExistsThenFocus(ev, storageUnit, kODPropContents, kTextEditorKind) )
				somSelf->StreamDataOut(ev, storageUnit->CreateView(ev), selRange, kIOAll);
			
			// Write out styled text for compatability with other parts
			if ( ODSUExistsThenFocus(ev, storageUnit, kODPropContents, gGlobals->fStyledTextKind) )
			{
				TempODStorageUnitView suView = storageUnit->CreateView(ev);
				ExportStyledTextData(ev, _fTextension, suView, (TOffsetRange*) selRange);
			}
			
			// Write out plain text for compatability with other parts
			if ( ODSUExistsThenFocus(ev, storageUnit, kODPropContents, gGlobals->fScrapTextValue) )
				somSelf->StreamDataOut(ev, storageUnit->CreateView(ev), selRange, kNoIOFlags);
		}
			
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		CleanseContentProperty
// Origin:		TextEditor
//
// Description:	This method is called during exteralization of the
//				part. The part should remove any value in the content property
//				that it cannot "accurately" write to.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__CleanseContentProperty
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit
			)
{
	SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
	SOMMethodDebug("TextEditor","Externalize");

	SOM_TRY
	
		ODULong numValues;
		ODULong index;
		
		storageUnit->Focus(ev, kODPropContents, kODPosUndefined, 
								kODNULL, 0, kODPosAll);
		
		numValues = storageUnit->CountValues(ev);
		
		for (index = numValues; index > 0; index--)
		{
			// Index from 1 to n through the values.
			storageUnit->Focus(ev, kODPropContents, kODPosUndefined, 
									kODNULL, index, kODPosUndefined);
			TempODValueType value = storageUnit->GetType(ev);
			
			ODBoolean remove = kODTrue;

			if ( (_fPreferredKind == gGlobals->fNativeData || _fPreferredKind == kODNullTypeToken) 
				 && ODISOStrEqual(value, kTextEditorKind) )
				remove = kODFalse;
			
			// Output styled text interchange data unless their preferred  
			// kind is specifically plain text.
			if ( _fPreferredKind != gGlobals->fImportedData 
				 && ODISOStrEqual(value, gGlobals->fStyledTextKind) )
				remove = kODFalse;
			
			if ( ODISOStrEqual(value, gGlobals->fScrapTextValue) )
				remove = kODFalse;

			if (remove)
				storageUnit->Remove(ev);
		}
			
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		CheckAndAddProperties
// Origin:		TextEditor
//
// Description:	This method is called to prepare the storage unit 
//				during the creation of stationery, and to verify 
//				that all the properties we need are present during
//				externalization. .
//
//				The part typically adds the default content property,
//				a preferred editor/viewer property (to aid in part binding), 
//				and a default name for the part.
//
// Note:		The function StorageUnitSetValue is a macro which
//				simplifies the use of ODByteArrary, which is required
//				in the StorageUnit interface. Look in StorUtil.h/cpp for
//				an example of using the ODByteArray struct.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__CheckAndAddProperties
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				storageUnit
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","CheckAndAddProperties");
	
	SOM_TRY
	
		// Create our content property.
		if ( !storageUnit->Exists(ev, kODPropContents, kODNULL, 0) )
			storageUnit->AddProperty(ev, kODPropContents);
		
		// Add values for all the kinds we support, in fidelity order.
		if ( !storageUnit->Exists(ev, kODPropContents, kTextEditorKind, 0) )
		{
			storageUnit->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, 0, kODPosUndefined);
			storageUnit->AddValue(ev, kTextEditorKind);
		}
		if ( !storageUnit->Exists(ev, kODPropContents, gGlobals->fStyledTextKind, 0) )
		{
			storageUnit->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, 0, kODPosUndefined);
			storageUnit->AddValue(ev, gGlobals->fStyledTextKind);
		}
		if ( !storageUnit->Exists(ev, kODPropContents, gGlobals->fScrapTextValue, 0) )
		{
			storageUnit->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, 0, kODPosUndefined);
			storageUnit->AddValue(ev, gGlobals->fScrapTextValue);
		}
	
		// Since we are setting up the preferred kind property, we just write
		// out our default "kind" for the editor. We can write out the user
		// chosen kind in the ExternalizeStateInfo method.
		if ( !storageUnit->Exists(ev, kODPropPreferredKind, kODISOStr, 0) )
			SetPreferredKind(ev, storageUnit, kTextEditorKind);

		// Add our display frame list.
		if (!storageUnit->Exists(ev, kODPropDisplayFrames, kODNULL, 0))
			storageUnit->AddProperty(ev, kODPropDisplayFrames);
		if (!storageUnit->Exists(ev, kODPropDisplayFrames, kODWeakStorageUnitRefs, 0))
		{
			storageUnit->Focus(ev, kODPropDisplayFrames, kODPosUndefined, kODNULL, 0, kODPosUndefined);
			storageUnit->AddValue(ev, kODWeakStorageUnitRefs);
		}
				
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		SetDirty
// Origin:		TextEditor
//
// Description:	This method is called by the part when the content or state of
//				the part has been modified by the user and the "Save" menu item
//				should be enabled.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__SetDirty
 			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","SetDirty");

	SOM_TRY
	
		// There is no need to repeatedly tell the draft we have changed;
		// once is sufficient.
		if ( !_fDirty && !_fReadOnlyStorage )
		{
			_fDirty = kODTrue;
			ODGetDraft(ev, somSelf)->SetChangedFromPrev(ev);
		}
				
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ReadPartInfo
// Origin:		ODPart
//
// Description:	When a frame is being internalized by the Draft, it will ask the
//				owner (part) to read in its info annotation on the frame.
//
//				The part uses a C++ helper class to encapsulate the information
//				we store with each frame, so we let it internalize itself from
//				the storage unit view.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	ODInfoType
SOMLINK		TextEditor__ReadPartInfo
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODFrame*					frame,
				ODStorageUnitView*			storageUnitView
			)
{
    SOMMethodDebug("TextEditor","Open");

	CFrameInfo* frameInfo = kODNULL;
		
	SOM_TRY

		frameInfo = new CFrameInfo(ODGetSession(ev, somSelf));
		
		TRY
			frameInfo->InitFromStorage(ev, storageUnitView);
		CATCH_ALL
			// Clean up the allocated memory.
			ODDeleteObject(frameInfo);
			// Alert the caller.
			RERAISE;
		ENDTRY
	
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return (ODInfoType)frameInfo;
}

//------------------------------------------------------------------------------
// Method:		WritePartInfo
// Origin:		ODPart
//
// Description:	When a frame is being externalized by the Draft, it will ask the
//				owner (part) to write out its info annotation on the frame.
//
//				The part uses a C++ helper class to encapsulate the information
//				we store with each frame, so we let it externalize itself to
//				the storage unit view.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__WritePartInfo
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODInfoType					partInfo,
				ODStorageUnitView*			storageUnitView
			)
{
    SOMMethodDebug("TextEditor","WritePartInfo");

	SOM_TRY
	
		// Tell our frame info class to write itself out into the pre-
		// focused storage unit.
		((CFrameInfo*) partInfo)->Externalize(ev, storageUnitView);
			
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ClonePartInfo
// Origin:		ODPart
//
// Description:	When a frame is being cloned by the Draft, it will ask the owner
//				(part) to clone its info annotation on the frame.
//
//				The part uses a C++ helper class to encapsulate the information
//				we store with each frame, so we let it clone itself to the
//				storage unit view.
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	void
SOMLINK		TextEditor__ClonePartInfo
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODDraftKey					key,
				ODInfoType					partInfo,
				ODStorageUnitView*			storageUnitView,
				ODFrame*					scopeFrame
			)
{
	SOMMethodDebug("TextEditor","ClonePartInfo");

	SOM_TRY
	
		// Tell our frame info class to write itself out into the pre-
		// focused storage unit.
		((CFrameInfo*) partInfo)->CloneInto(ev, key, storageUnitView, scopeFrame);
			
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		HasValidData
// Origin:		TextEditor
//------------------------------------------------------------------------------
#pragma segment TextEditorStorage

SOM_Scope	ODBoolean
SOMLINK		TextEditor__HasValidData
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnit*				su,
				ODBoolean*					requiresTranslation
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","HasValidData");
	
	ODBoolean canTranslate = kODFalse;
	
	SOM_TRY
	
		*requiresTranslation = kODFalse;
							
		for ( ODTypeToken kindToken = GetNextHighestFidelity();
			  kindToken != kODNullTypeToken;
			  kindToken = GetNextHighestFidelity(kindToken) )
		{
			if ( su->Exists(ev, kODPropContents, GetKindFromToken(ev, kindToken), 0) )
			{
				canTranslate = kODTrue;
				break;
			}
		}

		// Check for a text file value.
		if ( !canTranslate )
		{
			if ( su->Exists(ev, kODPropContents, gGlobals->fTextFileValue, 0) )
			{
				canTranslate = kODTrue;
			}
			else
			{
				ODSession* session = ODGetSession(ev, somSelf);
				ODTranslation* translation = session->GetTranslation(ev);
				
				// Get the number of values in the contents property.
				su->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, 0, kODPosAll);
				ODULong numValues = su->CountValues(ev);
				
				// Iterate from 1 to n through the values in the contents property.
				for (ODULong index = 1 ; index <= numValues ; index++)
				{
					su->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, index, kODPosUndefined);
	
					// Get the ISO type name for the value. The temp object
					// will automatically delete the returned value when this
					// scope is exited.
					TempODValueType foreignDataType = su->GetType(ev);
					
					// Verify that if there is a translator that recognizes the "foreign" data
					// type, that it can translate to a data type we understand.
					if ( translation->CanTranslate(ev, foreignDataType) == kODCanTranslate )
					{
						TempODTypeList validTypes = translation->GetTranslationOf(ev, foreignDataType);
						
						for ( TempODTypeListIterator type(ev, validTypes) ;
					 		  type.IsNotComplete() ; type.Next() )
						{
							ODTypeToken typeToken = session->Tokenize(ev, type);
							if ( IsKindSupported(ev, typeToken) )
							{
								canTranslate = kODTrue;
								*requiresTranslation = kODTrue;
								break;
							}
						}
					}
				}
			}
		}
	
	SOM_CATCH_ALL
		canTranslate = kODFalse;
	SOM_ENDTRY

	return canTranslate;
}

//------------------------------------------------------------------------------
// Method:		StreamDataOut
// Origin:		TextEditor
//------------------------------------------------------------------------------

SOM_Scope	void
SOMLINK		TextEditor__StreamDataOut
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnitView*			suView,
				ODPtr						selRange,
				ODUByte						ioFlags
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","StreamDataOut");

	SOM_TRY
	
		SUValueStream *stream = new SUValueStream;
		stream->InitValueStream(suView);
	
		if ( ioFlags != kNoIOFlags )
		{
			// We are writing out "native" Textension data
			
			CTextensionIOSuite *ioSuite = new CTextensionIOSuite;
			ioSuite->ITextensionIOSuite(false, stream);
			
			if ( selRange )
			{
				THROW_IF_ERROR(_fTextension->OutputRangeIOSuite((TOffsetRange*) selRange, 
																ioSuite, ioFlags));
			}
			else
			{
				TOffsetRange range2Save(0, _fTextension->CountCharsBytes());
				
				THROW_IF_ERROR(_fTextension->OutputRangeIOSuite(&range2Save, ioSuite, ioFlags));
			}
			
			ioSuite->Free();
			delete ioSuite;
		}
		else
		{
			// We are writing out plain TEXT data
			
			long bytes;
			long start; 
			
			if ( selRange )
			{
				bytes = ((TOffsetRange*)selRange)->End() - ((TOffsetRange*)selRange)->Start();
				start = ((TOffsetRange*)selRange)->Start();
			}
			else
			{
				bytes = _fTextension->CountCharsBytes();
				start = 0;
			}
			
			stream->Empty();
			stream->WriteBytes(_fTextension->GetCharPtr(start), bytes);
		}
			
		stream->Free();
		delete stream;
	
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		GetText
// Origin:		TextEditor
//------------------------------------------------------------------------------

SOM_Scope	ODBoolean
SOMLINK		TextEditor__GetText
			(
				SampleCode_TextEditor*	somSelf, 
				Environment*			ev,
				ODPlatformType			type,
				ODPtr*					textPtr,
				ODSize*					size 
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","StreamDataOut");

	ODBoolean result = kODFalse;
	
	SOM_TRY
	
		if ( type == 'TEXT' )
		{
			// Get the selection range.
			TOffsetRange curSelectionRange;
			_fTextension->GetSelectionRange(&curSelectionRange);
				
			// Get the text data.  The caller must not dispose it!
			if ( curSelectionRange.Start() == curSelectionRange.End() )
			{
				// No selection.
				*textPtr = _fTextension->GetCharPtr(0);
				*size    = _fTextension->CountCharsBytes();
			}
			else
			{
				*textPtr = _fTextension->GetCharPtr(curSelectionRange.Start());
				*size    = curSelectionRange.Len();
			}
			
			result = kODTrue;
		}
		
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

//------------------------------------------------------------------------------
// Method:		SetText
// Origin:		TextEditor
//------------------------------------------------------------------------------

SOM_Scope	ODBoolean
SOMLINK		TextEditor__SetText
			(
				SampleCode_TextEditor*	somSelf, 
				Environment*			ev,
				ODPlatformType			type,
				ODPtr					textPtr,
				ODSize					size
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","StreamDataOut");

	ODBoolean result = kODFalse;
	
	SOM_TRY
		
		CListIterator fiter(_fDisplayFrames);
		ODFrame* frame = ((CFrameProxy*) fiter.First())->GetFrame(ev);
		
		CFrameInfo* frameInfo = (CFrameInfo*) frame->GetPartInfo(ev);
		ODFacet* facet = frameInfo->GetActiveFacet();
	
		if ( !facet )
		{
			TempODFrameFacetIterator iter(ev, frame);
			facet = iter.First();
		}
		
		// Focus drawing environment so caret is drawn in the right place.
		CFocus initiateDrawing(ev, facet);

		WASSERT(type == 'STXT' || type == 'stxt');
		
		if ( !_fReadOnlyStorage )
		{
			// Get the selection range to replaced.
			TOffsetRange curSelectionRange;
			_fTextension->GetSelectionRange(&curSelectionRange);
			
			// If there's no selection, replace everything.
			if ( curSelectionRange.Start() == curSelectionRange.End() )
			{
				curSelectionRange.rangeStart = 0;
				curSelectionRange.rangeEnd = _fTextension->CountCharsBytes();
			}
			
			if ( type == 'STXT' || type == 'stxt' )
			{
				(void) InsertStyledTextData(ev, _fTextension, textPtr, size,
											&curSelectionRange);
				somSelf->HandleChange(ev);
				result = kODTrue;
			}
		}
	
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

//------------------------------------------------------------------------------
// Method:		StreamDataIn
// Origin:		TextEditor
//------------------------------------------------------------------------------

SOM_Scope	ODSLong
SOMLINK		TextEditor__StreamDataIn
			(
				SampleCode_TextEditor*		somSelf,
				Environment*				ev,
				ODStorageUnitView*			suView,
				ODPtr						selRange,
				ODUByte						ioFlags
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","StreamDataIn");

	ODSLong bytesRead = 0;

	SOM_TRY
	
		WASSERT(suView != kODNULL);
		
		// Determine where the data is coming from and create
		// a Textension stream object.
		
		SUValueStream* stream = new SUValueStream;
		stream->InitValueStream(suView);
		
		TRY
			if ( ioFlags != kNoIOFlags )
			{
				// Read in native Textension data.
				bytesRead = InsertNativeTextensionData(ev, _fTextension, stream, 
													   (TOffsetRange*) selRange);
			}
			else
			{
				// Read in generic Text data.
				bytesRead = InsertPlainTextData(ev, _fTextension, stream,
												(TOffsetRange*) selRange);
			}
		CATCH_ALL
			stream->Free();
			delete stream;
			RERAISE;
		ENDTRY
	
		stream->Free();
		delete stream;
		
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return bytesRead;
}

//------------------------------------------------------------------------------
// Method:		FulfillPromise
// Origin:		ODPart
//------------------------------------------------------------------------------

SOM_Scope	void
SOMLINK		TextEditor__FulfillPromise
			(
				SampleCode_TextEditor*		somSelf, 
				Environment*				ev,
				ODStorageUnitView*			promiseSUView
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","FulfillPromise");

	SOM_TRY
	
		THROW(kODErrNotImplemented);
	
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ExternalizeKinds
// Origin:		ODPart
//------------------------------------------------------------------------------

SOM_Scope	void
SOMLINK		TextEditor__ExternalizeKinds
			(
				SampleCode_TextEditor*		somSelf, 
				Environment*				ev,
				ODTypeList*					kindset
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","ExternalizeKinds");

	SOM_TRY
	
		if ( !_fReadOnlyStorage )
		{
			ODBoolean preferredKindWritten = kODFalse;

			// Get our storage unit.
			ODStorageUnit* storageUnit = somSelf->GetStorageUnit(ev);
			ODSession* session = ODGetSession(ev, somSelf);
			
			// Verify that the storage unit has the appropriate properties
			// and values to allow us to run. If not, add them.
			somSelf->CheckAndAddProperties(ev, storageUnit);
		
			// Verify that there are no "bogus" values in the Content
			// property.
			somSelf->CleanseContentProperty(ev, storageUnit);
			
			// Write out each of the kinds that we support from the list.
			// Note: We should externalize in fidelity order, but since the
			// content of TextEditor's types is about the same, we don't 
			// check the order.
			for ( TempODTypeListIterator kind(ev, kindset) ;
				  kind.IsNotComplete() ; kind.Next() )
			{
				ODTypeToken kindToken = session->Tokenize(ev, kind);
				
				if ( IsKindSupported(ev, kindToken) )
				{
					// Write out the part's content in the prescribed format.
					somSelf->ExternalizeContent(ev, storageUnit, kODNULL);
						//  Currently, ExternalizeContent (and CleanseContentProperty, 
						//    for that matter) are not set up to handle ExternalizeKinds 
						//    correctly because they don't allow you to specify the kinds 
						//	  to write.

					if ( kindToken == _fPreferredKind )
						preferredKindWritten = kODTrue;
				}
			}
	
			// Even if the kind set contains no types we support, we must at least
			// write out our current "preferred" kind.
			if ( !preferredKindWritten )
			{
				// Write out the part's preferred content kind.
				somSelf->ExternalizeContent(ev, storageUnit, kODNULL);
			}
			
			// Write out the part's state information.
			somSelf->ExternalizeStateInfo(ev, storageUnit, kODNULLKey, kODNULL);
		}

	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ChangeKind
// Origin:		ODPart
//------------------------------------------------------------------------------

SOM_Scope	void
SOMLINK		TextEditor__ChangeKind
			(
				SampleCode_TextEditor*		somSelf, 
				Environment*				ev,
				ODType						kind
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","ChangeKind");

	SOM_TRY

		ODTypeToken kindToken = ODGetSession(ev, somSelf)->Tokenize(ev, kind);
		
		// Make sure we support the kind we are being asked to use.
		if ( ODISOStrEqual(kind, gGlobals->fTextFileValue) )
		{
			kindToken = gGlobals->fNativeData;	
				// First, if the kind we were asked to use is the "file kind", then
				// use the data kind instead.  The file kind is intended to only
				// refer to incoming data in a drag-and-drop (or maybe some other
				// file-based binding) and be paired with an HFS kind with the 
				// file specification.  However, OpenDoc 1.0 shows this in its
				// Kind popup menu, and there is no way to prevent it from appearing
				// there.
		}
		else if ( !IsKindSupported(ev, kindToken) )
		{
			THROW(kODErrInvalidValueType);
		}
		
		// Get our storage unit.
		ODStorageUnit* storageUnit = somSelf->GetStorageUnit(ev);
		
		// We need to make sure the "Preferred Kind" property has the correct 
		// value right away so that OpenDoc knows our preferred kind has changed.
		if ( _fPreferredKind != kindToken )
		{
			_fPreferredKind = kindToken;
			
			SetPreferredKind(ev, storageUnit, GetKindFromToken(ev, kindToken));
			somSelf->SetDirty(ev);
			somSelf->Externalize(ev);
		}
	
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// Method:		TranslateForeignData
// Origin:		TextEditor
//------------------------------------------------------------------------------

SOM_Scope	ODBoolean
SOMLINK		TextEditor__TranslateForeignData
			(
				SampleCode_TextEditor*		somSelf, 
				Environment*				ev,
				ODStorageUnit*				storageUnit
			)
{
    SampleCode_TextEditorData *somThis = SampleCode_TextEditorGetData(somSelf);
    SOMMethodDebug("TextEditor","TranslateForeignData");

	return kODFalse;
}





